@@ -26,17 +26,34 @@ package picard.analysis;
 
 import htsjdk.samtools.util.CollectionUtil;
 import htsjdk.samtools.util.Log;
+import htsjdk.samtools.util.StringUtil;
+import org.apache.commons.lang3.StringUtils;
 import org.broadinstitute.barclay.argparser.Argument;
+import org.broadinstitute.barclay.argparser.CommandLineException;
+import org.broadinstitute.barclay.argparser.CommandLineParser;
 import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
 import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
 import picard.analysis.artifacts.CollectSequencingArtifactMetrics;
 import picard.analysis.directed.RnaSeqMetricsCollector;
 import picard.cmdline.CommandLineProgram;
-import picard.cmdline.programgroups.DiagnosticsAndQCProgramGroup;
 import picard.cmdline.StandardOptionDefinitions;
+import picard.cmdline.argumentcollections.RequiredOutputArgumentCollection;
+import picard.cmdline.programgroups.DiagnosticsAndQCProgramGroup;
+
 import java.io.File;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+import java.util.stream.Collectors;
 
 /**
  * Class that is designed to instantiate and execute multiple metrics programs that extend
@@ -59,8 +76,8 @@ public class CollectMultipleMetrics extends CommandLineProgram {
      * Includes a method for determining whether or not a Program explicitly needs a reference sequence (i.e. cannot be null)
      */
 
-    static final String USAGE_SUMMARY ="Collect multiple classes of metrics.  ";
-    static final String USAGE_DETAILS ="This 'meta-metrics' tool runs one or more of the metrics collection modules at the same" +
+    static final String USAGE_SUMMARY = "Collect multiple classes of metrics.";
+    static final String USAGE_DETAILS = "This 'meta-metrics' tool runs one or more of the metrics collection modules at the same" +
             " time to cut down on the time spent reading in data from input files. Available modules include " +
             "CollectAlignmentSummaryMetrics, CollectInsertSizeMetrics, QualityScoreDistribution,  MeanQualityByCycle, " +
             "CollectBaseDistributionByCycle, CollectGcBiasMetrics, RnaSeqMetrics, CollectSequencingArtifactMetrics" +
@@ -88,22 +105,25 @@ public class CollectMultipleMetrics extends CommandLineProgram {
             "      R=reference_sequence.fasta \\<br />" +
             "      PROGRAM=null \\<br />" +
             "      PROGRAM=QualityScoreDistribution \\<br />" +
-            "      PROGRAM=MeanQualityByCycle "+
+            "      PROGRAM=MeanQualityByCycle " +
             "</pre>" +
             "<hr />";
-    public static interface ProgramInterface {
 
-        /** By default, this method calls the
+    public interface ProgramInterface {
+
+        /**
+         * By default, this method calls the
          * {@link #makeInstance(String, String, File, File, Set, File, File, File, Set)} method
-         * without 'includeUnpaired' parameter. */
-        default  SinglePassSamProgram makeInstance(final String outbase,
-                                                   final String outext,
-                                                   final File input,
-                                                   final File reference,
-                                                   final Set<MetricAccumulationLevel> metricAccumulationLevel,
-                                                   final File dbSnp, final File intervals,
-                                                   final File refflat, Set<String> ignoreSequence,
-                                                   final boolean includeUnpaired) {
+         * without 'includeUnpaired' parameter.
+         */
+        default SinglePassSamProgram makeInstance(final String outbase,
+                                                  final String outext,
+                                                  final File input,
+                                                  final File reference,
+                                                  final Set<MetricAccumulationLevel> metricAccumulationLevel,
+                                                  final File dbSnp, final File intervals,
+                                                  final File refflat, Set<String> ignoreSequence,
+                                                  final boolean includeUnpaired) {
 
             return makeInstance(outbase, outext, input,
                     reference,
@@ -122,7 +142,7 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                           final File dbSnp,
                                           final File intervals,
                                           final File refflat,
-                                          final  Set<String> ignoreSequence);
+                                          final Set<String> ignoreSequence);
 
         default boolean needsReferenceSequence() {
             return false;
@@ -137,9 +157,8 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         }
     }
 
-    public static enum Program implements ProgramInterface {
+    public enum Program implements ProgramInterface {
         CollectAlignmentSummaryMetrics {
-
             @Override
             public boolean supportsMetricAccumulationLevel() {
                 return true;
@@ -154,9 +173,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final CollectAlignmentSummaryMetrics program = new CollectAlignmentSummaryMetrics();
-                program.OUTPUT = new File(outbase + ".alignment_summary_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".alignment_summary_metrics" + outext));
 
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
@@ -170,7 +189,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         CollectInsertSizeMetrics {
-
             @Override
             public boolean supportsMetricAccumulationLevel() {
                 return true;
@@ -185,9 +203,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final CollectInsertSizeMetrics program = new CollectInsertSizeMetrics();
-                program.OUTPUT = new File(outbase + ".insert_size_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection( new File(outbase + ".insert_size_metrics" + outext));
                 program.Histogram_FILE = new File(outbase + ".insert_size_histogram.pdf");
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
@@ -201,7 +219,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         QualityScoreDistribution {
-
             @Override
             public SinglePassSamProgram makeInstance(final String outbase, final String outext, final File input,
                                                      final File reference,
@@ -209,9 +226,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final QualityScoreDistribution program = new QualityScoreDistribution();
-                program.OUTPUT = new File(outbase + ".quality_distribution_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".quality_distribution_metrics" + outext));
                 program.CHART_OUTPUT = new File(outbase + ".quality_distribution.pdf");
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
@@ -224,7 +241,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         MeanQualityByCycle {
-
             @Override
             public SinglePassSamProgram makeInstance(final String outbase,
                                                      final String outext,
@@ -234,9 +250,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final MeanQualityByCycle program = new MeanQualityByCycle();
-                program.OUTPUT = new File(outbase + ".quality_by_cycle_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".quality_by_cycle_metrics" + outext));
                 program.CHART_OUTPUT = new File(outbase + ".quality_by_cycle.pdf");
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
@@ -249,7 +265,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         CollectBaseDistributionByCycle {
-
             @Override
             public SinglePassSamProgram makeInstance(final String outbase,
                                                      final String outext,
@@ -259,9 +274,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final CollectBaseDistributionByCycle program = new CollectBaseDistributionByCycle();
-                program.OUTPUT = new File(outbase + ".base_distribution_by_cycle_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".base_distribution_by_cycle_metrics" + outext));
                 program.CHART_OUTPUT = new File(outbase + ".base_distribution_by_cycle.pdf");
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
@@ -274,7 +289,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         CollectGcBiasMetrics {
-
             @Override
             public boolean needsReferenceSequence() {
                 return true;
@@ -294,9 +308,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final CollectGcBiasMetrics program = new CollectGcBiasMetrics();
-                program.OUTPUT = new File(outbase + ".gc_bias.detail_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".gc_bias.detail_metrics" + outext));
                 program.SUMMARY_OUTPUT = new File(outbase + ".gc_bias.summary_metrics" + outext);
                 program.CHART_OUTPUT = new File(outbase + ".gc_bias.pdf");
                 program.INPUT = input;
@@ -315,7 +329,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         RnaSeqMetrics {
-
             @Override
             public boolean needsRefflatFile() {
                 return true;
@@ -335,9 +348,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final CollectRnaSeqMetrics program = new CollectRnaSeqMetrics();
-                program.OUTPUT       = new File(outbase + ".rna_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".rna_metrics" + outext));
                 program.CHART_OUTPUT = new File(outbase + ".rna_coverage.pdf");
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
@@ -354,7 +367,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         CollectSequencingArtifactMetrics {
-
             @Override
             public boolean needsReferenceSequence() {
                 return true;
@@ -369,7 +381,7 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
 
                 return makeInstance(outbase, outext, input,
                         reference,
@@ -380,6 +392,7 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                         ignoreSequence,
                         false);
             }
+
             @Override
             public SinglePassSamProgram makeInstance(final String outbase,
                                                      final String outext,
@@ -389,10 +402,10 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence,
+                                                     final Set<String> ignoreSequence,
                                                      final boolean includeUnpaired) {
                 final CollectSequencingArtifactMetrics program = new CollectSequencingArtifactMetrics();
-                program.OUTPUT = new File(outbase);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase));
                 program.FILE_EXTENSION = outext;
                 program.DB_SNP = dbSnp;
                 program.INTERVALS = intervals;
@@ -408,7 +421,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
         },
 
         CollectQualityYieldMetrics {
-
             @Override
             public SinglePassSamProgram makeInstance(final String outbase,
                                                      final String outext,
@@ -418,9 +430,9 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                                                      final File dbSnp,
                                                      final File intervals,
                                                      final File refflat,
-                                                     final  Set<String> ignoreSequence) {
+                                                     final Set<String> ignoreSequence) {
                 final CollectQualityYieldMetrics program = new CollectQualityYieldMetrics();
-                program.OUTPUT = new File(outbase + ".quality_yield_metrics" + outext);
+                program.output = new RequiredOutputArgumentCollection(new File(outbase + ".quality_yield_metrics" + outext));
                 // Generally programs should not be accessing these directly but it might make things smoother
                 // to just set them anyway. These are set here to make sure that in case of a the derived class
                 // overrides
@@ -446,17 +458,17 @@ public class CollectMultipleMetrics extends CommandLineProgram {
             doc = "Base name of output files.")
     public String OUTPUT;
 
-    //create the default accumulation level as a variable.
+    // create the default accumulation level as a variable.
     // We'll use this to init the command-line arg and for validation later.
     private final Set<MetricAccumulationLevel> accumLevelDefault = CollectionUtil.makeSet(MetricAccumulationLevel.ALL_READS);
 
-    @Argument(shortName="LEVEL",
-            doc="The level(s) at which to accumulate metrics.")
+    @Argument(shortName = "LEVEL",
+            doc = "The level(s) at which to accumulate metrics.")
     public Set<MetricAccumulationLevel> METRIC_ACCUMULATION_LEVEL = new HashSet<>(accumLevelDefault);
 
     @Argument(shortName = "EXT",
-            doc="Append the given file extension to all metric file names (ex. OUTPUT.insert_size_metrics.EXT). None if null",
-            optional=true)
+            doc = "Append the given file extension to all metric file names (ex. OUTPUT.insert_size_metrics.EXT). None if null",
+            optional = true)
     public String FILE_EXTENSION = null;
 
     @Argument(doc = "Set of metrics programs to apply during the pass through the SAM file.")
@@ -478,12 +490,12 @@ public class CollectMultipleMetrics extends CommandLineProgram {
             optional = true)
     public File DB_SNP;
 
-    @Argument(doc="Gene annotations in refFlat form.  " +
+    @Argument(doc = "Gene annotations in refFlat form.  " +
             "Format described here: http://genome.ucsc.edu/goldenPath/gbdDescriptionsOld.html#RefFlat",
             optional = true)
     public File REF_FLAT;
 
-    @Argument(doc="If a read maps to a sequence specified with this option, " +
+    @Argument(doc = "If a read maps to a sequence specified with this option, " +
             "all the bases in the read are counted as ignored bases.",
             optional = true)
     public Set<String> IGNORE_SEQUENCE = new HashSet<>();
@@ -491,9 +503,21 @@ public class CollectMultipleMetrics extends CommandLineProgram {
     @Argument(shortName = "UNPAIRED",
             doc = "Include unpaired reads in CollectSequencingArtifactMetrics. " +
                     "If set to true then all paired reads will be included as well - " +
-            "MINIMUM_INSERT_SIZE and MAXIMUM_INSERT_SIZE will be ignored in CollectSequencingArtifactMetrics.")
+                    "MINIMUM_INSERT_SIZE and MAXIMUM_INSERT_SIZE will be ignored in CollectSequencingArtifactMetrics.")
     public boolean INCLUDE_UNPAIRED = false;
 
+    @Argument(doc="extra arguments to the various tools can be specified using the following format:" +
+            "<PROGRAM>::<ARGUMENT_AND_VALUE> where <PROGRAM> is one of the programs specified in PROGRAM, " +
+            "and <ARGUMENT_AND_VALUE> are the argument and value that you'd like to specify as you would on the command line. " +
+            "For example, to change the HISTOGRAM_WIDTH in CollectInsertSizeMetrics to 200, use:\n " +
+            "\"EXTRA_ARGUMENT=CollectInsertSizeMetrics::HISTOGRAM_WIDTH=200\"\n " +
+            "or, in the new parser:" +
+            "--EXTRA_ARGUMENT \"CollectInsertSizeMetrics::--HISTOGRAM_WIDTH 200\"\n " +
+            "(Quotes are required to avoid the shell from separating this into two arguments.) " +
+            "Note that the following arguments cannot be modified on a per-program level: INPUT, REFERENCE_SEQUENCE, ASSUME_SORTED, and STOP_AFTER. " +
+            "Providing them in an EXTRA_ARGUMENT will _not_ result in an error, but they will be silently ignored. " , optional = true)
+    public List<String> EXTRA_ARGUMENT = null;
+
     /**
      * Contents of PROGRAM set is transferred to this set during command-line validation, so that an outside
      * developer can invoke this class programmatically and provide alternative Programs to run by calling
@@ -503,11 +527,6 @@ public class CollectMultipleMetrics extends CommandLineProgram {
 
     private static final Log log = Log.getInstance(CollectMultipleMetrics.class);
 
-    // Stock main method
-    public static void main(final String[] args) {
-        new CollectMultipleMetrics().instanceMainWithExit(args);
-    }
-
     @Override
     protected String[] customCommandLineValidation() {
         if (PROGRAM.isEmpty()) {
@@ -533,6 +552,8 @@ public class CollectMultipleMetrics extends CommandLineProgram {
             OUTPUT = OUTPUT.substring(0, OUTPUT.length() - 1);
         }
 
+        final Map<ProgramInterface, List<String>> additionalArguments = processAdditionalArguments(EXTRA_ARGUMENT);
+
         final List<SinglePassSamProgram> programs = new ArrayList<>();
         for (final ProgramInterface program : programsToRun) {
             if (program.needsReferenceSequence() && REFERENCE_SEQUENCE == null) {
@@ -557,11 +578,53 @@ public class CollectMultipleMetrics extends CommandLineProgram {
                     REF_FLAT,
                     IGNORE_SEQUENCE,
                     INCLUDE_UNPAIRED);
+
+            if (additionalArguments.containsKey(program)) {
+                final CommandLineParser commandLineParser = getCommandLineParser(instance);
+                final boolean success = commandLineParser.parseArguments(System.err, additionalArguments.get(program).toArray(new String[0]));
+                if (!success) {
+                    throw new CommandLineException("Failed to parse arguments ["+ String.join(",", additionalArguments.get(program))+ "] for " + program);
+                }
+                additionalArguments.remove(program);
+            }
+
             instance.setDefaultHeaders(getDefaultHeaders());
             programs.add(instance);
         }
+        if (!additionalArguments.isEmpty()) {
+            throw new CommandLineException("EXTRA_ARGUMENT values were provided, but corresponding PROGRAM wasn't requested:" +
+                    additionalArguments.entrySet().stream().map(e -> e.getKey().toString() + "::" + e.getValue().toString()).collect(Collectors.joining()));
+        }
         SinglePassSamProgram.makeItSo(INPUT, REFERENCE_SEQUENCE, ASSUME_SORTED, STOP_AFTER, programs);
 
         return 0;
     }
+
+    private static Map<ProgramInterface, List<String>> processAdditionalArguments(final List<String> arguments) {
+        final Map<ProgramInterface, List<String>> map = new HashMap<>();
+        final Pattern pattern = Pattern.compile("(?<program>.*)::(?<argumentAndValue>.+?)( +(?<optionalValue>.+))?");
+        for (String str : arguments) {
+            final Matcher matcher = pattern.matcher(str);
+            if (!matcher.matches()) {
+                throw new CommandLineException("couldn't understand EXTRA_ARGUMENT " + str +
+                        " it doesn't conform to the form '<PROGRAM>::<ARGUMENT_AND_VALUE>'.");
+            }
+            final String programName = matcher.group("program");
+            final ProgramInterface program;
+            try {
+                program = Program.valueOf(programName);
+            } catch (IllegalArgumentException e) {
+                throw new CommandLineException("Couldn't find program with value " + programName, e);
+            }
+            final String argumentAndValue = matcher.group("argumentAndValue");
+
+            map.computeIfAbsent(program, (k) -> new ArrayList<>()).add(argumentAndValue);
+
+            final String optionalValue = matcher.group("optionalValue");
+            if (!StringUtils.isEmpty(optionalValue)) {
+                map.get(program).add(optionalValue);
+            }
+        }
+        return map;
+    }
 }
