@@ -32,11 +32,12 @@ import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.IntervalList;
 import htsjdk.samtools.util.Log;
 import htsjdk.samtools.util.StringUtil;
+import htsjdk.utils.ValidationUtils;
+import org.broadinstitute.barclay.argparser.Argument;
+import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
 import org.broadinstitute.barclay.argparser.ExperimentalFeature;
 import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
-import org.broadinstitute.barclay.argparser.Argument;
-import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
 import picard.cmdline.programgroups.DiagnosticsAndQCProgramGroup;
 import picard.filter.CountingFilter;
 import picard.filter.CountingPairedFilter;
@@ -61,9 +62,9 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
             "with non-zero (>0) coverage." +
             "<p>Note: Metrics labeled as percentages are actually expressed as fractions!</p>" +
             "<h4>Usage Example:</h4>" +
-            "<pre>"  +
+            "<pre>" +
             "java -jar picard.jar CollectWgsMetricsWithNonZeroCoverage \\<br /> " +
-            "      I=input.bam \\<br /> "+
+            "      I=input.bam \\<br /> " +
             "      O=collect_wgs_metrics.txt \\<br /> " +
             "      CHART=collect_wgs_metrics.pdf  \\<br /> " +
             "      R=reference_sequence.fasta " +
@@ -83,11 +84,15 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
 
     private SamReader samReader = null;
 
-    /** Metrics for evaluating the performance of whole genome sequencing experiments. */
+    /**
+     * Metrics for evaluating the performance of whole genome sequencing experiments.
+     */
     public static class WgsMetricsWithNonZeroCoverage extends WgsMetrics {
-        public enum Category { WHOLE_GENOME, NON_ZERO_REGIONS }
+        public enum Category {WHOLE_GENOME, NON_ZERO_REGIONS}
 
-        /** One of either WHOLE_GENOME or NON_ZERO_REGIONS */
+        /**
+         * One of either WHOLE_GENOME or NON_ZERO_REGIONS
+         */
         @MergeByAssertEquals
         public Category CATEGORY;
 
@@ -98,6 +103,7 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
         public WgsMetricsWithNonZeroCoverage(final IntervalList intervals,
                                              final Histogram<Integer> highQualityDepthHistogram,
                                              final Histogram<Integer> unfilteredDepthHistogram,
+                                             final double pctExcludedByAdapter,
                                              final double pctExcludedByMapq,
                                              final double pctExcludedByDupes,
                                              final double pctExcludedByPairing,
@@ -108,15 +114,11 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
                                              final int coverageCap,
                                              final Histogram<Integer> unfilteredBaseQHistogram,
                                              final int sampleSize) {
-            super(intervals, highQualityDepthHistogram, unfilteredDepthHistogram, pctExcludedByMapq, pctExcludedByDupes, pctExcludedByPairing, pctExcludedByBaseq,
+            super(intervals, highQualityDepthHistogram, unfilteredDepthHistogram, pctExcludedByAdapter, pctExcludedByMapq, pctExcludedByDupes, pctExcludedByPairing, pctExcludedByBaseq,
                     pctExcludedByOverlap, pctExcludedByCapping, pctTotal, coverageCap, unfilteredBaseQHistogram, sampleSize);
         }
     }
 
-    public static void main(final String[] args) {
-        new CollectWgsMetrics().instanceMainWithExit(args);
-    }
-
     @Override
     protected SamReader getSamReader() {
         if (this.samReader == null) {
@@ -159,6 +161,7 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
     protected WgsMetrics generateWgsMetrics(final IntervalList intervals,
                                             final Histogram<Integer> highQualityDepthHistogram,
                                             final Histogram<Integer> unfilteredDepthHistogram,
+                                            final double pctExcludedByAdapter,
                                             final double pctExcludedByMapq,
                                             final double pctExcludedByDupes,
                                             final double pctExcludedByPairing,
@@ -173,6 +176,7 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
                 intervals,
                 highQualityDepthHistogram,
                 unfilteredDepthHistogram,
+                pctExcludedByAdapter,
                 pctExcludedByMapq,
                 pctExcludedByDupes,
                 pctExcludedByPairing,
@@ -187,7 +191,7 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
 
     @Override
     protected WgsMetricsCollector getCollector(final int coverageCap, final IntervalList intervals) {
-        assert(coverageCap == this.collector.coverageCap);
+        ValidationUtils.validateArg(coverageCap == this.collector.coverageCap, () -> "coverageCap has to be the same as the internal coverageCap, found " + coverageCap + " and " + this.collector.coverageCap);
         return this.collector;
     }
 
@@ -196,7 +200,7 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
         Histogram<Integer> highQualityDepthHistogramNonZero;
 
         public WgsMetricsWithNonZeroCoverageCollector(final CollectWgsMetricsWithNonZeroCoverage metrics,
-                final int coverageCap, final IntervalList intervals) {
+                                                      final int coverageCap, final IntervalList intervals) {
             super(metrics, coverageCap, intervals);
         }
 
@@ -204,13 +208,14 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
         public void addToMetricsFile(final MetricsFile<WgsMetrics, Integer> file,
                                      final boolean includeBQHistogram,
                                      final CountingFilter dupeFilter,
+                                     final CountingFilter adapterFilter,
                                      final CountingFilter mapqFilter,
                                      final CountingPairedFilter pairFilter) {
             highQualityDepthHistogram = getDepthHistogram();
             highQualityDepthHistogramNonZero = getDepthHistogramNonZero();
 
             // calculate metrics the same way as in CollectWgsMetrics
-            final WgsMetricsWithNonZeroCoverage metrics = (WgsMetricsWithNonZeroCoverage) getMetrics(dupeFilter, mapqFilter, pairFilter);
+            final WgsMetricsWithNonZeroCoverage metrics = (WgsMetricsWithNonZeroCoverage) getMetrics(dupeFilter, adapterFilter, mapqFilter, pairFilter);
             metrics.CATEGORY = WgsMetricsWithNonZeroCoverage.Category.WHOLE_GENOME;
 
             // set count of the coverage-zero bin to 0 and re-calculate metrics
@@ -218,7 +223,7 @@ public class CollectWgsMetricsWithNonZeroCoverage extends CollectWgsMetrics {
             highQualityDepthHistogramArray[0] = 0;
             unfilteredDepthHistogramArray[0] = 0;
 
-            final WgsMetricsWithNonZeroCoverage metricsNonZero = (WgsMetricsWithNonZeroCoverage) getMetrics(dupeFilter, mapqFilter, pairFilter);
+            final WgsMetricsWithNonZeroCoverage metricsNonZero = (WgsMetricsWithNonZeroCoverage) getMetrics(dupeFilter, adapterFilter, mapqFilter, pairFilter);
             metricsNonZero.CATEGORY = WgsMetricsWithNonZeroCoverage.Category.NON_ZERO_REGIONS;
 
             file.addMetric(metrics);
