@@ -89,6 +89,12 @@ public abstract class CommandLineProgram {
     private static String PROPERTY_CONVERT_LEGACY_COMMAND_LINE = "picard.convertCommandLine";
     private static Boolean useLegacyParser;
 
+    /**
+     * CommandLineProgramProperties oneLineSummary attribute must be shorted than this in order to maintain
+     * reasonable help output formatting.
+     */
+    public static int MAX_ALLOWABLE_ONE_LINE_SUMMARY_LENGTH = 120;
+
     @Argument(doc="One or more directories with space available to be used by this program for temporary storage of working files",
             common=true, optional=true)
     public List<File> TMP_DIR = new ArrayList<>();
@@ -240,11 +246,15 @@ public abstract class CommandLineProgram {
             SAMFileWriterImpl.setDefaultMaxRecordsInRam(MAX_RECORDS_IN_RAM);
         }
 
+        final boolean defaultHTSJDKIndexCreation = SAMFileWriterFactory.getDefaultCreateIndexWhileWriting();
         if (CREATE_INDEX) {
             SAMFileWriterFactory.setDefaultCreateIndexWhileWriting(true);
         }
 
-        SAMFileWriterFactory.setDefaultCreateMd5File(CREATE_MD5_FILE);
+        final boolean defaultMD5Creation = SAMFileWriterFactory.getDefaultCreateMd5File();
+        if (CREATE_MD5_FILE) {
+            SAMFileWriterFactory.setDefaultCreateMd5File(CREATE_MD5_FILE);
+        }
 
         for (final File f : TMP_DIR) {
             // Intentionally not checking the return values, because it may be that the program does not
@@ -294,6 +304,9 @@ public abstract class CommandLineProgram {
         try {
             ret = doWork();
         } finally {
+            // restore the default to eliminate problems in downstream code caused by setting CREATE_INDEX=true above
+            SAMFileWriterFactory.setDefaultCreateIndexWhileWriting(defaultHTSJDKIndexCreation);
+            SAMFileWriterFactory.setDefaultCreateMd5File(defaultMD5Creation);
             try {
                 // Emit the time even if program throws
                 if (!QUIET) {
@@ -377,14 +390,20 @@ public abstract class CommandLineProgram {
      */
     public CommandLineParser getCommandLineParser() {
         if (commandLineParser == null) {
-            commandLineParser = useLegacyParser(getClass()) ?
-                        new LegacyCommandLineArgumentParser(this) :
-                        new CommandLineArgumentParser(this,
-                            Collections.EMPTY_LIST,
-                            new HashSet<>(Collections.singleton(CommandLineParserOptions.APPEND_TO_COLLECTIONS)));
+            commandLineParser = getCommandLineParser(this);
         }
         return commandLineParser;
     }
+    /**
+     * @return Return a newly minted command line parser for the provided object.
+     */
+    static public CommandLineParser getCommandLineParser(Object o) {
+        return useLegacyParser(o.getClass()) ?
+                        new LegacyCommandLineArgumentParser(o) :
+                        new CommandLineArgumentParser(o,
+                            Collections.EMPTY_LIST,
+                            new HashSet<>(Collections.singleton(CommandLineParserOptions.APPEND_TO_COLLECTIONS)));
+    }
 
     /**
      * Return true if the Picard command line parser should be used in place of the Barclay command line parser,
