@@ -23,6 +23,7 @@
  */
 package picard.sam;
 
+import htsjdk.samtools.CigarOperator;
 import htsjdk.samtools.SAMFileHeader.SortOrder;
 import htsjdk.samtools.SAMProgramRecord;
 import htsjdk.samtools.SAMRecord;
@@ -145,7 +146,7 @@ public class MergeBamAlignment extends CommandLineProgram {
     public final PGTagArgumentCollection pgTagArgumentCollection = new PGTagArgumentCollection();
 
     @Argument(shortName = "UNMAPPED",
-            doc = "Original SAM or BAM file of unmapped reads, which must be in queryname order.")
+            doc = "Original SAM or BAM file of unmapped reads, which must be in queryname order.  Reads MUST be unmapped.")
     public File UNMAPPED_BAM;
 
     @Argument(shortName = "ALIGNED",
@@ -256,9 +257,14 @@ public class MergeBamAlignment extends CommandLineProgram {
             "alignment is filtered out for some reason. For all strategies, ties are resolved arbitrarily.")
     public PrimaryAlignmentStrategy PRIMARY_ALIGNMENT_STRATEGY = PrimaryAlignmentStrategy.BestMapq;
 
-    @Argument(doc = "For paired reads, soft clip the 3' end of each read if necessary so that it does not extend past the 5' end of its mate.")
+    @Argument(doc = "For paired reads, clip the 3' end of each read if necessary so that it does not extend past the 5' end of its mate.  " +
+            "Clipping will be either soft or hard clipping, depending on CLIP_OVERLAPPING_READS_OPERATOR setting. " +
+            "Hard clipped bases and their qualities will be stored in the XB and XQ tags respectively.")
     public boolean CLIP_OVERLAPPING_READS = true;
 
+    @Argument(doc = "If true, hard clipping will be applied to overlapping reads.  By default, soft clipping is used.")
+    public boolean HARD_CLIP_OVERLAPPING_READS = false;
+
     @Argument(doc = "If false, do not write secondary alignments to output.")
     public boolean INCLUDE_SECONDARY_ALIGNMENTS = true;
 
@@ -275,7 +281,9 @@ public class MergeBamAlignment extends CommandLineProgram {
     @Argument(doc = "List of Sequence Records tags that must be equal (if present) in the reference dictionary and in the aligned file. Mismatching tags will cause an error if in this list, and a warning otherwise.")
     public List<String> MATCHING_DICTIONARY_TAGS = SAMSequenceDictionary.DEFAULT_DICTIONARY_EQUAL_TAG;
 
-    @Argument(doc = "How to deal with alignment information in reads that are being unmapped (e.g. due to cross-species contamination.) Currently ignored unless UNMAP_CONTAMINANT_READS = true", optional = true)
+    @Argument(doc = "How to deal with alignment information in reads that are being unmapped (e.g. due to cross-species contamination.) " +
+            "Currently ignored unless UNMAP_CONTAMINANT_READS = true. Note that the DO_NOT_CHANGE strategy will actually reset the cigar and set the mapping quality on unmapped reads since otherwise" +
+            "the result will be an invalid record. To force no change use the DO_NOT_CHANGE_INVALID strategy.", optional = true)
     public AbstractAlignmentMerger.UnmappingReadStrategy UNMAPPED_READ_STRATEGY = AbstractAlignmentMerger.UnmappingReadStrategy.DO_NOT_CHANGE;
 
     @Override
@@ -340,20 +348,22 @@ public class MergeBamAlignment extends CommandLineProgram {
         }
 
         final SamAlignmentMerger merger = new SamAlignmentMerger(UNMAPPED_BAM, OUTPUT,
-                REFERENCE_SEQUENCE, prod, CLIP_ADAPTERS, IS_BISULFITE_SEQUENCE,
+                referenceSequence.getReferenceFile(), prod, CLIP_ADAPTERS, IS_BISULFITE_SEQUENCE,
                 ALIGNED_READS_ONLY, ALIGNED_BAM, MAX_INSERTIONS_OR_DELETIONS,
                 ATTRIBUTES_TO_RETAIN, ATTRIBUTES_TO_REMOVE, READ1_TRIM, READ2_TRIM,
                 READ1_ALIGNED_BAM, READ2_ALIGNED_BAM, EXPECTED_ORIENTATIONS, SORT_ORDER,
                 PRIMARY_ALIGNMENT_STRATEGY.newInstance(), ADD_MATE_CIGAR, UNMAP_CONTAMINANT_READS,
                 MIN_UNCLIPPED_BASES, UNMAPPED_READ_STRATEGY, MATCHING_DICTIONARY_TAGS);
+
         merger.setClipOverlappingReads(CLIP_OVERLAPPING_READS);
+        merger.setHardClipOverlappingReads(HARD_CLIP_OVERLAPPING_READS);
         merger.setMaxRecordsInRam(MAX_RECORDS_IN_RAM);
         merger.setKeepAlignerProperPairFlags(ALIGNER_PROPER_PAIR_FLAGS);
         merger.setIncludeSecondaryAlignments(INCLUDE_SECONDARY_ALIGNMENTS);
         merger.setAttributesToReverse(ATTRIBUTES_TO_REVERSE);
         merger.setAttributesToReverseComplement(ATTRIBUTES_TO_REVERSE_COMPLEMENT);
         merger.setAddPGTagToReads(pgTagArgumentCollection.ADD_PG_TAG_TO_READS);
-        merger.mergeAlignment(REFERENCE_SEQUENCE);
+        merger.mergeAlignment(referenceSequence.getReferenceFile());
         merger.close();
 
         return 0;
