@@ -24,7 +24,9 @@
 
 package picard.sam.SamErrorMetric;
 
+import com.google.common.annotations.VisibleForTesting;
 import htsjdk.samtools.SAMFileHeader;
+import htsjdk.samtools.SAMRecord;
 import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.SamReader;
 import htsjdk.samtools.SamReaderFactory;
@@ -32,12 +34,23 @@ import htsjdk.samtools.metrics.MetricsFile;
 import htsjdk.samtools.reference.ReferenceSequenceFileWalker;
 import htsjdk.samtools.reference.SamLocusAndReferenceIterator;
 import htsjdk.samtools.reference.SamLocusAndReferenceIterator.SAMLocusAndReference;
-import htsjdk.samtools.util.*;
+import htsjdk.samtools.util.AbstractRecordAndOffset;
+import htsjdk.samtools.util.CloseableIterator;
+import htsjdk.samtools.util.CollectionUtil;
+import htsjdk.samtools.util.IOUtil;
+import htsjdk.samtools.util.IntervalList;
+import htsjdk.samtools.util.Locus;
+import htsjdk.samtools.util.Log;
+import htsjdk.samtools.util.PeekableIterator;
+import htsjdk.samtools.util.ProgressLogger;
+import htsjdk.samtools.util.QualityUtil;
+import htsjdk.samtools.util.SamLocusIterator;
 import htsjdk.tribble.util.ParsingUtils;
 import htsjdk.variant.variantcontext.VariantContext;
 import htsjdk.variant.vcf.VCFFileReader;
 import org.broadinstitute.barclay.argparser.Argument;
 import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
 import picard.cmdline.CommandLineProgram;
 import picard.cmdline.StandardOptionDefinitions;
@@ -45,7 +58,15 @@ import picard.cmdline.programgroups.DiagnosticsAndQCProgramGroup;
 
 import java.io.File;
 import java.io.IOException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Random;
+import java.util.Set;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
 
@@ -74,7 +95,7 @@ import java.util.stream.Collectors;
                 "<p>" +
                 "The resulting metric file will be named according to a provided prefix and a suffix which is generated " +
                 " automatically according to the error metric. " +
-                "The tool cal collect multiple metrics in a single pass and there should be hardly any " +
+                "The tool can collect multiple metrics in a single pass and there should be hardly any " +
                 "performance loss when specifying multiple metrics at the same time; the default includes a " +
                 "large collection of metrics. \n" +
                 "<p>" +
@@ -87,9 +108,18 @@ import java.util.stream.Collectors;
         oneLineSummary = "Program to collect error metrics on bases stratified in various ways.",
         programGroup = DiagnosticsAndQCProgramGroup.class
 )
+@DocumentedFeature
 public class CollectSamErrorMetrics extends CommandLineProgram {
+
+    // =====================================================================
+
+    private static final int MAX_DIRECTIVES = ReadBaseStratification.Stratifier.values().length + 1;
+    private static final Log log = Log.getInstance(CollectSamErrorMetrics.class);
+
+    // =====================================================================
+
     @Argument(shortName = StandardOptionDefinitions.INPUT_SHORT_NAME, doc = "Input SAM or BAM file.")
-    public File INPUT;
+    public String INPUT;
 
     @Argument(shortName = StandardOptionDefinitions.OUTPUT_SHORT_NAME, doc = "Base name for output files. Actual file names will be " +
             "generated from the basename and suffixes from the ERROR and STRATIFIER by adding a '.' and then " +
@@ -126,7 +156,9 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
             "OVERLAPPING_ERROR:READ_ORDINALITY",
             "OVERLAPPING_ERROR:READ_ORDINALITY:CYCLE",
             "OVERLAPPING_ERROR:READ_ORDINALITY:HOMOPOLYMER",
-            "OVERLAPPING_ERROR:READ_ORDINALITY:GC_CONTENT");
+            "OVERLAPPING_ERROR:READ_ORDINALITY:GC_CONTENT",
+            "INDEL_ERROR"
+    );
 
     @Argument(doc = "A fake argument used to show the options of ERROR (in ERROR_METRICS).", optional = true)
     public ErrorType ERROR_VALUE;
@@ -136,9 +168,10 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
 
     @Argument(shortName = "V", doc = "VCF of known variation for sample. program will skip over polymorphic sites in this VCF and " +
             "avoid collecting data on these loci.")
-    public File VCF;
+    public String VCF;
 
-    @Argument(shortName = "L", doc = "Region(s) to limit analysis to. Supported formats are VCF or interval_list. Will intersect inputs if multiple are given. ", optional = true)
+    @Argument(shortName = "L", doc = "Region(s) to limit analysis to. Supported formats are VCF or interval_list. Will *intersect* inputs if multiple are given. " +
+            "When this argument is supplied, the VCF provided must be *indexed*.", optional = true)
     public List<File> INTERVALS;
 
     @Argument(shortName = StandardOptionDefinitions.MINIMUM_MAPPING_QUALITY_SHORT_NAME, doc = "Minimum mapping quality to include read.")
@@ -160,6 +193,53 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
     @Argument(shortName = "P", doc = "The probability of selecting a locus for analysis (for downsampling).", optional = true)
     public double PROBABILITY = 1;
 
+    @Argument(
+            fullName = "PROGRESS_STEP_INTERVAL",
+            doc = "The interval between which progress will be displayed.",
+            optional = true
+    )
+    public int PROGRESS_STEP_INTERVAL = 100000;
+
+    @Argument(
+            fullName = "INTERVAL_ITERATOR",
+            doc = "Iterate through the file assuming it consists of a pre-created subset interval of the full genome.  " +
+                    "This enables fast processing of files with reads at disperate parts of the genome.  " +
+                    "Requires that the provided VCF file is indexed. ",
+            optional = true
+    )
+    public boolean INTERVAL_ITERATOR = false;
+
+    // =====================================================================
+
+    /** Random object from which to pull pseudo-random numbers.  Initialized in {@link #initializeAggregationState()}.*/
+    private Random random;
+
+    /** Aggregator list to store errors.  Initialized in {@link #initializeAggregationState()}.*/
+    private Collection<BaseErrorAggregation> aggregatorList;
+
+    /** Logger with which to keep the user apprised of our progress.  Initialized in {@link #initializeAggregationState()}.*/
+    private ProgressLogger progressLogger;
+
+    /** Count of the total number of loci visited.  Initialized in {@link #initializeAggregationState()}.*/
+    private long nTotalLoci;
+
+    /** Count of the number of skipped loci.  Initialized in {@link #initializeAggregationState()}.*/
+    private long nSkippedLoci;
+
+    /** Count of the number of processed loci.  Initialized in {@link #initializeAggregationState()}.*/
+    private long nProcessedLoci;
+
+    /** The {@link SAMSequenceDictionary} for the reference, variants, and reads being processed. */
+    private SAMSequenceDictionary sequenceDictionary;
+
+    /** A {@link VCFFileReader} to read in variants when running in {@link #INTERVAL_ITERATOR} mode. */
+    private VCFFileReader vcfFileReader;
+
+    /** A {@link PeekableIterator<VariantContext>} to read in variants when running in default mode. */
+    private PeekableIterator<VariantContext> vcfIterator;
+
+    // =====================================================================
+
     @Override
     protected boolean requiresReference() {
         return true;
@@ -211,73 +291,100 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
         }
     }
 
-    private static final int MAX_DIRECTIVES = ReadBaseStratification.Stratifier.values().length + 1;
-    private static final Log log = Log.getInstance(CollectSamErrorMetrics.class);
-
-    @Override
-    protected int doWork() {
-
-        final Random random = new Random(42);
-
-        final ProgressLogger progressLogger = new ProgressLogger(log, 100000);
-        long nTotalLoci = 0;
-        long nSkippedLoci = 0;
-        long nProcessedLoci = 0;
-
-        IOUtil.assertFileIsReadable(INPUT);
-        IOUtil.assertFilesAreReadable(INTERVALS);
-
-        final Collection<BaseErrorAggregation> aggregatorList = getAggregatorList();
-        // Open up the input resources:
-        try (
-                final SamReader sam = SamReaderFactory.makeDefault().open(INPUT);
-                final ReferenceSequenceFileWalker referenceSequenceFileWalker = new ReferenceSequenceFileWalker(REFERENCE_SEQUENCE);
-                final PeekableIterator<VariantContext> vcfIterator = new PeekableIterator<>(
-                        VCF == null ? Collections.emptyIterator() : new VCFFileReader(VCF, true).iterator())
-        ) {
-
-            final SAMSequenceDictionary sequenceDictionary = referenceSequenceFileWalker.getSequenceDictionary();
-            if (sam.getFileHeader().getSortOrder() != SAMFileHeader.SortOrder.coordinate) {
-                throw new PicardException("Input BAM must be sorted by coordinate");
+    /**
+     * Initialize the source for {@link VariantContext} information.
+     * If running in {@link #INTERVAL_ITERATOR} mode, will initialize the {@link #vcfFileReader}.
+     * Otherwise, will initialize the {@link #vcfIterator}.
+     */
+    private void initializeVcfDataSource() throws IOException {
+        if ( INTERVAL_ITERATOR ) {
+            vcfFileReader = new VCFFileReader(IOUtil.getPath(VCF), false);
+            // Make sure we can query our file for interval mode:
+            if (!vcfFileReader.isQueryable()) {
+                throw new PicardException("Cannot query VCF File!  VCF Files must be queryable!  Please index input VCF and re-run.");
             }
+        }
+        else {
+            vcfIterator = new PeekableIterator<>(
+                    VCF == null ? Collections.emptyIterator() : new VCFFileReader(IOUtil.getPath(VCF), false).iterator());
+        }
+    }
 
-            sequenceDictionary.assertSameDictionary(sam.getFileHeader().getSequenceDictionary());
+    /**
+     * Check if the given locus overlaps a variant from our variant source.
+     *
+     * If running in {@link #INTERVAL_ITERATOR} mode:
+     * This will query the input variant file for overlapping variants, rather than just iterating through them.
+     * It will be a little slower than the normal iteration, and much faster if a subset of a genome is used.
+     *
+     * If running in default mode:
+     * Process our input data by iterating through input variants and reads.
+     * Useful for going through whole genomes / exomes, but slow for sets of reads not beginning at the start of the
+     * given variant file.
+     * This slowness is because {@link CollectSamErrorMetrics} by default iterates through both the reads and the
+     * variants sequentially from start to finish.  In the case your region of interest begins much later in the genome
+     * (e.g. chr20:145000) then you will have to wait for the variant file to be traversed from the start to your
+     * locus of interest.
+     *
+     * NOTE: This method HAS SIDE EFFECTS for both the {@link #vcfFileReader} and {@link #vcfIterator}.
+     *
+     * @return {@code true} if a variant overlaps the given locus.  {@code false} otherwise.
+     */
+    private boolean checkLocusForVariantOverlap(final SamLocusIterator.LocusInfo locusInfo) {
+        final boolean returnValue;
+        if (INTERVAL_ITERATOR) {
+            returnValue = checkLocus(vcfFileReader, locusInfo);
+            if ( returnValue ) {
+                log.debug("Locus overlaps a known variant: " + locusInfo);
+            }
+        }
+        else {
+            returnValue = advanceIteratorAndCheckLocus(vcfIterator, locusInfo, sequenceDictionary);
+            if ( returnValue ) {
+                log.debug(String.format("Locus overlaps a known variant from VCF: %s -> %s", locusInfo.toString(),
+                        vcfIterator.peek().toStringWithoutGenotypes()));
+            }
+        }
+        return returnValue;
+    }
 
-            final IntervalList regionOfInterest = getIntervals(sequenceDictionary);
+    /**
+     * Close the variant data source.
+     * If running in {@link #INTERVAL_ITERATOR} mode will close the {@link #vcfFileReader};
+     * If running in default mode will close the {@link #vcfIterator};
+     */
+    private void closeVcfDataSource() {
+        if (INTERVAL_ITERATOR) {
+            vcfFileReader.close();
+        }
+        else {
+            vcfIterator.close();
+        }
+    }
 
-            log.info("Getting SamLocusIterator");
+    private int processData() {
+        try (
+                final SamReader sam = SamReaderFactory.makeDefault()
+                        .referenceSequence(REFERENCE_SEQUENCE)
+                        .open(IOUtil.getPath(INPUT));
+                final ReferenceSequenceFileWalker referenceSequenceFileWalker = new ReferenceSequenceFileWalker(REFERENCE_SEQUENCE)
+        ) {
+            // Initialize our variants:
+            initializeVcfDataSource();
 
-            final SamLocusIterator samLocusIterator = new SamLocusIterator(sam, regionOfInterest);
+            // Go through our reads and variants now:
+            final SamLocusAndReferenceIterator iterator = createSamLocusAndReferenceIterator(sam, referenceSequenceFileWalker);
 
-            // Now go through and compare information at each of these sites
-            samLocusIterator.setEmitUncoveredLoci(false);
-            samLocusIterator.setMappingQualityScoreCutoff(MIN_MAPPING_Q);
-            samLocusIterator.setQualityScoreCutoff(MIN_BASE_Q);
-
-            log.info("Using " + aggregatorList.size() + " aggregators.");
-
-            aggregatorList.forEach(la ->
-                    IOUtil.assertFileIsWritable(new File(OUTPUT + la.getSuffix())));
-
-            // iterate over loci
-            log.info("Starting iteration over loci");
-
-            final SamLocusAndReferenceIterator iterator = new SamLocusAndReferenceIterator(referenceSequenceFileWalker, samLocusIterator);
-
-            // This hasNext() call has side-effects. It loads up the index and makes sure that the iterator is really ready for
-            // action. Calling this allows for the logging to be more accurate.
-            iterator.hasNext();
             log.info("Really starting iteration now.");
-
             for (final SAMLocusAndReference info : iterator) {
                 if (random.nextDouble() > PROBABILITY) {
                     continue;
                 }
                 nTotalLoci++;
 
-                // while there is a next (non-filtered) variant and it is before the locus, advance the pointer.
-                if (advanceIteratorAndCheckLocus(vcfIterator, info.getLocus(), sequenceDictionary)) {
-                    log.debug(String.format("Skipping locus from VCF: %s", vcfIterator.peek().toStringWithoutGenotypes()));
+                // Check if a variant overlaps the current locus:
+                if ( checkLocusForVariantOverlap(info.getLocus()) ) {
+                    log.debug("Skipping overlapping locus.");
                     nSkippedLoci++;
                     continue;
                 }
@@ -293,20 +400,124 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
                 }
             }
 
-        } catch (IOException e) {
-            log.error(e, "A problem occurred:");
+        } catch (final IOException e) {
+            log.error(e, "A problem occurred:", e.getMessage());
             return 1;
         }
-        log.info("Iteration complete, generating metric files");
-
-        aggregatorList.forEach(this::writeMetricsFileForAggregator);
-
-        log.info(String.format("Examined %d loci, Processed %d loci, Skipped %d loci.\n" +
-                "Computation took %d seconds.", nTotalLoci, nProcessedLoci, nSkippedLoci, progressLogger.getElapsedSeconds()));
+        finally {
+            // Close our data source:
+            closeVcfDataSource();
+        }
 
         return 0;
     }
 
+    /**
+     * Creates the {@link SamLocusAndReferenceIterator} object to use to traverse the input files.
+     * Also initializes the {@link #sequenceDictionary} and performs some checks on the output
+     * files in {@link #aggregatorList} to make sure we can write our output.
+     * @param sam an open {@link SamReader} from which to pull reads.
+     * @param referenceSequenceFileWalker an open {@link ReferenceSequenceFileWalker} from which to get reference sequence information.
+     * @return An open {@link SamLocusAndReferenceIterator} ready to iterate.
+     */
+    private SamLocusAndReferenceIterator createSamLocusAndReferenceIterator(final SamReader sam, final ReferenceSequenceFileWalker referenceSequenceFileWalker) {
+
+        sequenceDictionary = referenceSequenceFileWalker.getSequenceDictionary();
+        if (sam.getFileHeader().getSortOrder() != SAMFileHeader.SortOrder.coordinate) {
+            throw new PicardException("Input BAM must be sorted by coordinate");
+        }
+
+        // Make sure our reference and reads have the same sequence dictionary:
+        sequenceDictionary.assertSameDictionary(sam.getFileHeader().getSequenceDictionary());
+
+        final IntervalList regionOfInterest = getIntervals(sequenceDictionary);
+
+        log.info("Getting SamLocusIterator");
+
+        final SamLocusIterator samLocusIterator = new SamLocusIterator(sam, regionOfInterest);
+
+        // We want to know about indels:
+        samLocusIterator.setIncludeIndels(true);
+
+        // Now go through and compare information at each of these sites
+        samLocusIterator.setEmitUncoveredLoci(false);
+        samLocusIterator.setMappingQualityScoreCutoff(MIN_MAPPING_Q);
+        samLocusIterator.setQualityScoreCutoff(MIN_BASE_Q);
+
+        log.info("Using " + aggregatorList.size() + " aggregators.");
+
+        aggregatorList.forEach(la ->
+                IOUtil.assertFileIsWritable(new File(OUTPUT + la.getSuffix())));
+
+        // iterate over loci
+        log.info("Starting iteration over loci");
+
+        final SamLocusAndReferenceIterator iterator = new SamLocusAndReferenceIterator(referenceSequenceFileWalker, samLocusIterator);
+
+        // This hasNext() call has side-effects. It loads up the index and makes sure that
+        // the iterator is really ready for
+        // action. Calling this allows for the logging to be more accurate.
+        iterator.hasNext();
+        return iterator;
+    }
+
+    /**
+     * Initializes the counts, random object, and iterators for this class.
+     *
+     * Specifically, the following fields are initialized:
+     *
+     *   random
+     *   aggregatorList
+     *   progressLogger
+     *   nTotalLoci
+     *   nSkippedLoci
+     *   nProcessedLoci
+     */
+    private void initializeAggregationState() {
+        random = new Random(42);
+
+        aggregatorList = getAggregatorList();
+
+        progressLogger = new ProgressLogger(log, PROGRESS_STEP_INTERVAL);
+        nTotalLoci = 0;
+        nSkippedLoci = 0;
+        nProcessedLoci = 0;
+    }
+
+    @Override
+    protected int doWork() {
+
+        // Initialize our iteration:
+        initializeAggregationState();
+
+        // Make sure we can read our files:
+        try {
+            IOUtil.assertFileIsReadable(IOUtil.getPath(INPUT));
+            IOUtil.assertFileIsReadable(IOUtil.getPath(VCF));
+            IOUtil.assertFileIsReadable(REFERENCE_SEQUENCE);
+            IOUtil.assertFilesAreReadable(INTERVALS);
+        }
+        catch (final IOException e) {
+            log.error(e, "A problem occurred:", e.getMessage());
+            return 1;
+        }
+
+        // Process our data based on how we will be iterating:
+        final int returnValue = processData();
+
+        // Check if we had an error and if so, immediately return
+        // (to preserve old functionality):
+        if (returnValue == 0) {
+            log.info("Iteration complete, generating metric files");
+
+            aggregatorList.forEach(this::writeMetricsFileForAggregator);
+
+            log.info(String.format("Examined %d loci, Processed %d loci, Skipped %d loci.\n" +
+                    "Computation took %d seconds.", nTotalLoci, nProcessedLoci, nSkippedLoci, progressLogger.getElapsedSeconds()));
+        }
+        return returnValue;
+    }
+
     /**
      * Given a "full" BaseCalculator, get all of its metrics and put them in the appropriate Metrics file.
      *
@@ -343,13 +554,138 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
         return vcfIterator.hasNext() && CompareVariantContextToLocus(sequenceDictionary, vcfIterator.peek(), locus) == 0;
     }
 
+    /**
+     * Map of previously seen deletion records, associated with the locus they have been last seen
+     */
+    private final HashMap<SAMRecord, SamLocusIterator.LocusInfo> previouslySeenDeletions = new HashMap<>();
+
+    /**
+     * Current locus for taking care of deleting deletion records from the above map
+     */
+    private SamLocusIterator.LocusInfo currentLocus = null;
+
+    /**
+     * Checks if the same record has been seen at the previous locus already, thereby determining
+     * whether or not a deletion has already been processed. Note that calling this method will
+     * have the side effect of signaling that the record is processed at this location.
+     *
+     * @param deletionRao The RecordAndOffset to be checked
+     * @param locusInfo   The LocusInfo to determine the current locus
+     * @return True, if the record has been seen at the previous locus. False, if it has not yet been seen.
+     */
+    @VisibleForTesting
+    protected boolean processDeletionLocus(final SamLocusIterator.RecordAndOffset deletionRao, final SamLocusIterator.LocusInfo locusInfo) {
+        if (currentLocus == null) {
+            currentLocus = locusInfo;
+        }
+
+        // Check if we have moved to a new locus
+        else if (!currentLocus.withinDistanceOf(locusInfo, 0)) {
+            // If yes, remove all entries that have not been seen in the previous locus
+            currentLocus = locusInfo;
+            previouslySeenDeletions.entrySet().removeIf(entry -> !entry.getValue().withinDistanceOf(currentLocus, 1));
+        }
+        if (previouslySeenDeletions.containsKey(deletionRao.getRecord())) {
+            previouslySeenDeletions.put(deletionRao.getRecord(), currentLocus);
+            return true;
+        }
+        previouslySeenDeletions.put(deletionRao.getRecord(), currentLocus);
+        return false;
+    }
+
+    /**
+     * Stratifies the current RecordAndOffset. In case isDeletionRecord is true, the record is checked for whether or not
+     * this deletion has already been processed, as it will be populated for each locus in the reference
+     *
+     * @param aggregatorList The aggregators to add the bases to
+     * @param rao            The ReadAndOffset object
+     * @param info           The SAMLocusAndReference object
+     */
+    private void addRecordAndOffset(final Collection<BaseErrorAggregation> aggregatorList, final SamLocusIterator.RecordAndOffset rao, final SAMLocusAndReference info) {
+        // If deletion has been processed already, skip it
+        if (rao.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Deletion && processDeletionLocus(rao, info.getLocus()))
+            return;
+
+        for (final BaseErrorAggregation aggregation : aggregatorList) {
+            aggregation.addBase(rao, info);
+        }
+    }
+
+    /**
+     * Compares a VariantContext to a Locus providing information regarding possible overlap, or relative location
+     *
+     * @param dictionary     The {@link SAMSequenceDictionary} to use for ordering the sequences
+     * @param variantContext the {@link VariantContext} to compare
+     * @param locus          the {@link Locus} to compare
+     * @return negative if variantContext comes before locus (with no overlap)
+     * zero if variantContext and locus overlap
+     * positive if variantContext comes after locus (with no overlap)
+     * <p/>
+     * if variantContext and locus are in the same contig the return value will be the number of bases apart they are,
+     * otherwise it will be MIN_INT/MAX_INT
+     */
+    public static int CompareVariantContextToLocus(final SAMSequenceDictionary dictionary, final VariantContext variantContext, final Locus locus) {
+
+        final int indexDiff = dictionary.getSequenceIndex(variantContext.getContig()) - locus.getSequenceIndex();
+        if (indexDiff != 0) {
+            return indexDiff < 0 ? Integer.MIN_VALUE : Integer.MAX_VALUE;
+        }
+
+        //same SequenceIndex, can compare by genomic position
+        if (locus.getPosition() < variantContext.getStart()) {
+            return variantContext.getStart() - locus.getPosition();
+        }
+        if (locus.getPosition() > variantContext.getEnd()) {
+            return variantContext.getEnd() - locus.getPosition();
+        }
+        return 0;
+    }
+
+    /**
+     * If there's a variant at the locus return true, otherwise false.
+     * <p>
+     * HAS SIDE EFFECTS!!! Queries the vcfFileReader
+     *
+     * @param vcfFileReader      a {@link VCFFileReader} to query for the given locus
+     * @param locusInfo          a {@link SamLocusIterator.LocusInfo} at which to examine the variants
+     * @return true if there's a variant over the locus, false otherwise.
+     */
+    private static boolean checkLocus(final VCFFileReader vcfFileReader, final SamLocusIterator.LocusInfo locusInfo) {
+        boolean overlaps = false;
+
+        if (locusInfo != null) {
+            try (final CloseableIterator<VariantContext> vcfIterator = vcfFileReader.query(locusInfo)) {
+
+                while (vcfIterator.hasNext()) {
+                    if (vcfIterator.next().isFiltered()) {
+                        continue;
+                    }
+                    overlaps = true;
+                    break;
+                }
+            }
+        }
+        return overlaps;
+    }
+
     /**
      * Iterate over the different records in the locus and add bases to aggregators
      */
     private void addLocusBases(final Collection<BaseErrorAggregation> aggregatorList, final SAMLocusAndReference info) {
-        info.getRecordAndOffsets()
-                .forEach(rao -> aggregatorList
-                        .forEach(l -> l.addBase(rao, info)));
+        // Matching bases
+        for (final SamLocusIterator.RecordAndOffset rao : info.getRecordAndOffsets()) {
+            addRecordAndOffset(aggregatorList, rao, info);
+        }
+
+        // Deleted bases
+        for (final SamLocusIterator.RecordAndOffset deletionRao : info.getLocus().getDeletedInRecord()) {
+            addRecordAndOffset(aggregatorList, deletionRao, info);
+        }
+
+        // Inserted bases
+        for (final SamLocusIterator.RecordAndOffset insertionRao : info.getLocus().getInsertedInRecord()) {
+            addRecordAndOffset(aggregatorList, insertionRao, info);
+        }
     }
 
     /**
@@ -377,7 +713,7 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
     }
 
     /**
-     * Reads Interprets intervals from the input INTERVALS, if there's a file with that name, it opens the file, otherwise
+     * Interprets intervals from the input INTERVALS, if there's a file with that name, it opens the file, otherwise
      * it tries to parse it, checks that their dictionaries all agree with the input SequenceDictionary and returns the
      * intersection of all the lists.
      *
@@ -408,34 +744,6 @@ public class CollectSamErrorMetrics extends CommandLineProgram {
         return regionOfInterest;
     }
 
-    /**
-     * Compares a VariantContext to a Locus providing information regarding possible overlap, or relative location
-     *
-     * @param dictionary     The {@link SAMSequenceDictionary} to use for ordering the sequences
-     * @param variantContext the {@link VariantContext} to compare
-     * @param locus          the {@link Locus} to compare
-     * @return negative if variantContext comes before locus (with no overlap)
-     * zero if variantContext and locus overlap
-     * positive if variantContext comes after locus (with no overlap)
-     * <p/>
-     * if variantContext and locus are in the same contig the return value will be the number of bases apart they are,
-     * otherwise it will be MIN_INT/MAX_INT
-     */
-    public static int CompareVariantContextToLocus(final SAMSequenceDictionary dictionary, final VariantContext variantContext, final Locus locus) {
-
-        final int indexDiff = dictionary.getSequenceIndex(variantContext.getContig()) - locus.getSequenceIndex();
-        if (indexDiff != 0) {
-            return indexDiff < 0 ? Integer.MIN_VALUE : Integer.MAX_VALUE;
-        }
-
-        //same SequenceIndex, can compare by genomic position
-        if (locus.getPosition() < variantContext.getStart())
-            return variantContext.getStart() - locus.getPosition();
-        if (locus.getPosition() > variantContext.getEnd())
-            return variantContext.getEnd() - locus.getPosition();
-        return 0;
-    }
-
     /**
      * Parses a "Directive" of the form "ERROR,STRATIFIER,STRATIFIER...etc." into a {@link BaseErrorAggregation} consisting of the appropriate
      * {@link BaseCalculator} and the {@link ReadBaseStratification.CollectionStratifier CollectionStratifier} constructed from the various
