@@ -25,11 +25,12 @@ package picard.sam.SamErrorMetric;
 
 import com.google.common.cache.Cache;
 import com.google.common.cache.CacheBuilder;
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMTag;
+import htsjdk.samtools.*;
 import htsjdk.samtools.reference.SamLocusAndReferenceIterator.SAMLocusAndReference;
+import htsjdk.samtools.util.AbstractRecordAndOffset;
 import htsjdk.samtools.util.Lazy;
-import htsjdk.samtools.util.SamLocusIterator.*;
+import htsjdk.samtools.util.Log;
+import htsjdk.samtools.util.SamLocusIterator.RecordAndOffset;
 import htsjdk.samtools.util.SequenceUtil;
 import org.broadinstitute.barclay.argparser.CommandLineParser;
 import picard.sam.markduplicates.util.OpticalDuplicateFinder;
@@ -47,9 +48,11 @@ import java.util.stream.Stream;
 
 /**
  * Classes, methods, and enums that deal with the stratification of read bases and reference information.
- *
  */
 public class ReadBaseStratification {
+
+    private static final Log log = Log.getInstance(CollectSamErrorMetrics.class);
+
     // This variable has to be set using setLongHomopolymer _before_ the first call to binnedHomopolymerStratifier.get()
     // if you need different binned Homopolymers with different LONG_HOMOPOLYMER values, you'll need to make your own
     // instances.
@@ -76,8 +79,6 @@ public class ReadBaseStratification {
      * The main interface for a stratifier.
      * The Stratifier needs to be able to ake a {@link htsjdk.samtools.util.SamLocusIterator.RecordAndOffset RecordAndOffset}
      * and return a value of type T. It also needs to have a suffix for automatically generating metrics file suffixes.
-     *
-     * @param <T>
      */
     public interface RecordAndOffsetStratifier<T extends Comparable<T>> {
         // The method that stratifies a base in a read (provided by RecordAndOffset) into a type T.
@@ -195,7 +196,7 @@ public class ReadBaseStratification {
     public static class CollectionStratifier implements RecordAndOffsetStratifier {
         public CollectionStratifier(final Collection<RecordAndOffsetStratifier<?>> stratifiers) {
 
-            if (stratifiers.isEmpty()){
+            if (stratifiers.isEmpty()) {
                 throw new IllegalArgumentException("Must construct with a non-empty collection of stratifiers.");
             }
 
@@ -210,12 +211,13 @@ public class ReadBaseStratification {
 
             stratifier = linkedListStratifiers.remove();
         }
+
         final RecordAndOffsetStratifier stratifier;
 
         @Override
         public Comparable stratify(final RecordAndOffset recordAndOffset, final SAMLocusAndReference locusInfo) {
 
-            return stratifier.stratify(recordAndOffset,locusInfo);
+            return stratifier.stratify(recordAndOffset, locusInfo);
         }
 
         @Override
@@ -224,17 +226,15 @@ public class ReadBaseStratification {
         }
     }
 
-
     /**
      * A factory for generating "pair" stratifier instances from two stratifiers and a string
      *
-     * @param <T>    the type of the left Stratifier
-     * @param <S>    the type of the right Stratifier
-     * @param leftStratifier a {@link RecordAndOffsetStratifier<T>} to use
-     * @param rightStratifier a {@link RecordAndOffsetStratifier<S>} to use
-     * @param suffix the suffix to use for the new stratifier
-     *
-     * @return an instance of {@link  PairStratifier>} that will stratify according to both <code>leftStratifier</code>
+     * @param <T>             the type of the left Stratifier
+     * @param <S>             the type of the right Stratifier
+     * @param leftStratifier  a {@link RecordAndOffsetStratifier} to use
+     * @param rightStratifier a {@link RecordAndOffsetStratifier} to use
+     * @param suffix          the suffix to use for the new stratifier
+     * @return an instance of {@link  PairStratifier} that will stratify according to both <code>leftStratifier</code>
      * and <code>rightStratifier</code>
      */
     public static <T extends Comparable<T>, S extends Comparable<S>> PairStratifier<T, S> PairStratifierFactory(
@@ -293,7 +293,9 @@ public class ReadBaseStratification {
                                              final SAMLocusAndReference locusInfo) {
 
             final Integer hpLength = homoPolymerLengthStratifier.stratify(recordAndOffset, locusInfo);
-            if (hpLength == null) return null;
+            if (hpLength == null) {
+                return null;
+            }
 
             return hpLength < longHomopolymer ? LongShortHomopolymer.SHORT_HOMOPOLYMER : LongShortHomopolymer.LONG_HOMOPOLYMER;
 
@@ -428,7 +430,7 @@ public class ReadBaseStratification {
     }
 
     /**
-     * Stratifies according to the overall mismatches (from NM) that the read has against the reference, NOT
+     * Stratifies according to the overall mismatches (from {@link SAMTag#NM}) that the read has against the reference, NOT
      * including the current base.
      */
     public static class MismatchesInReadStratifier implements RecordAndOffsetStratifier<Integer> {
@@ -479,6 +481,71 @@ public class ReadBaseStratification {
         }
     }
 
+    /**
+     * Stratifies according to the number of matching cigar operators (from CIGAR string) that the read has.
+     */
+    public static class CigarOperatorsInReadStratifier extends RecordStratifier<Integer> {
+
+        private CigarOperator operator;
+
+        public CigarOperatorsInReadStratifier(final CigarOperator op) {
+            operator = op;
+        }
+
+        @Override
+        public Integer stratify(final SAMRecord samRecord) {
+            return stratifyCigarOperatorsInRead(samRecord, operator);
+        }
+
+        @Override
+        public String getSuffix() {
+            return "cigar_elements_" + operator.name() + "_in_read";
+        }
+    }
+
+    /**
+     * Stratifies according to the number of indel bases (from CIGAR string) that the read has.
+     */
+    public static class IndelsInReadStratifier extends RecordStratifier<Integer> {
+
+        /**
+         * Returns the number of bases associated with I and D CIGAR elements.
+         * @param samRecord The read to investigate
+         * @return The number of bases associated with I and D CIGAR elements, or null if the evaluation of either
+         *         operation caused an error
+         */
+        @Override
+        public Integer stratify(final SAMRecord samRecord) {
+            final Integer insertedBasesInRead = stratifyCigarOperatorsInRead(samRecord, CigarOperator.I);
+            final Integer deletedBasesInRead = stratifyCigarOperatorsInRead(samRecord, CigarOperator.D);
+            if (insertedBasesInRead == null || deletedBasesInRead == null) {
+                return null;
+            }
+            return insertedBasesInRead + deletedBasesInRead;
+        }
+
+        @Override
+        public String getSuffix() {
+            return "indels_in_read";
+        }
+    }
+
+    /**
+     * Stratifies according to the length of an insertion or deletion.
+     */
+    public static class IndelLengthStratifier implements RecordAndOffsetStratifier {
+
+        @Override
+        public Integer stratify(final RecordAndOffset recordAndOffset, final SAMLocusAndReference locusInfo) {
+            return stratifyIndelLength(recordAndOffset, locusInfo);
+        }
+
+        @Override
+        public String getSuffix() {
+            return "indel_length";
+        }
+    }
+
 
     /* ************ Instances of stratifiers so that they do not need to be allocated every time ****************/
 
@@ -537,7 +604,7 @@ public class ReadBaseStratification {
      * <p>
      * This is done with a Lazy wrapper since it requires access to {@link #LONG_HOMOPOLYMER} which can be set with {@link #setLongHomopolymer(int)}.
      */
-    public static final Lazy<PairStratifier<LongShortHomopolymer, Pair<Character, Character>>> binnedHomopolymerStratifier = new Lazy<>(() -> PairStratifierFactory( new LongShortHomopolymerStratifier(LONG_HOMOPOLYMER), preDiNucleotideStratifier,
+    public static final Lazy<PairStratifier<LongShortHomopolymer, Pair<Character, Character>>> binnedHomopolymerStratifier = new Lazy<>(() -> PairStratifierFactory(new LongShortHomopolymerStratifier(LONG_HOMOPOLYMER), preDiNucleotideStratifier,
             "binned_length_homopolymer_and_following_ref_base"));
 
     /**
@@ -581,6 +648,11 @@ public class ReadBaseStratification {
      */
     public static final RecordAndOffsetStratifier<ReadOrdinality> readOrdinalityStratifier = wrapStaticReadFunction(ReadOrdinality::of, "read_ordinality");
 
+    /**
+     * Stratifies bases into their read's Proper-pairedness
+     */
+    public static final RecordAndOffsetStratifier<ProperPaired> readPairednessStratifier = wrapStaticReadFunction(ProperPaired::of, "pair_proper");
+
     /**
      * Stratifies bases into their read's Direction (i.e. forward or reverse)
      */
@@ -606,6 +678,11 @@ public class ReadBaseStratification {
      */
     public static final RecordAndOffsetStratifier<Integer> insertLengthStratifier = wrapStaticReadFunction(ReadBaseStratification::stratifyInsertLength, "insert_length");
 
+    /**
+     * Stratifies into the number of soft-clipped bases that the read has in its alignment, or {@value NOT_ALIGNED_ERROR} if not aligned.
+     */
+    public static final RecordAndOffsetStratifier<Integer> softClipsLengthStratifier = wrapStaticReadFunction(ReadBaseStratification::stratifySoftClippedBases, "softclipped_bases");
+
     /**
      * Stratifies into the base-quality of the base under consideration
      */
@@ -633,6 +710,26 @@ public class ReadBaseStratification {
      */
     public static final NsInReadStratifier nsInReadStratifier = new NsInReadStratifier();
 
+    /**
+     * Stratify by Insertions in the read cigars.
+     */
+    public static final CigarOperatorsInReadStratifier insertionsInReadStratifier = new CigarOperatorsInReadStratifier(CigarOperator.I);
+
+    /**
+     * Stratify by Deletions in the read cigars.
+     */
+    public static final CigarOperatorsInReadStratifier deletionsInReadStratifier = new CigarOperatorsInReadStratifier(CigarOperator.D);
+
+    /**
+     * Stratify by Indels in the read cigars.
+     */
+    public static final IndelsInReadStratifier indelsInReadStratifier = new IndelsInReadStratifier();
+
+    /**
+     * Stratifies into the number of bases in an insertion
+     */
+    public static final IndelLengthStratifier indelLengthStratifier = new IndelLengthStratifier();
+
     /* *************** enums **************/
 
     /**
@@ -646,30 +743,36 @@ public class ReadBaseStratification {
      */
     enum Stratifier implements CommandLineParser.ClpEnum {
         ALL(() -> nonStratifier, "Puts all bases in the same stratum."),
-        GC_CONTENT(() -> gcContentStratifier, "Stratifies bases according to the gc content of their read."),
-        READ_ORDINALITY(() -> readOrdinalityStratifier, "Stratifies bases according to their read ordinality (i.e. first or second)."),
-        READ_BASE(() -> currentReadBaseStratifier, "Stratifies bases by the read in the original reading direction."),
-        READ_DIRECTION(() -> readDirectionStratifier, "Stratifies bases to +/- based on the alignment direction of the read."),
-        PAIR_ORIENTATION(() -> readOrientationStratifier, "Stratifies bases into F1R2/F2R1 according to their reads orientation and ordinality. Assumes reads are \"innies\"."),
-        REFERENCE_BASE(() -> referenceBaseStratifier, "Stratifies bases by the read-directed reference base."),
-        PRE_DINUC(() -> preDiNucleotideStratifier, "Stratifies bases by the read base at the previous cycle, and the current reference base."),
-        POST_DINUC(() -> postDiNucleotideStratifier, "Stratifies bases by the read base at the previous cycle, and the current reference base."),
-        HOMOPOLYMER_LENGTH(() -> homoPolymerLengthStratifier, "Stratifies bases based on the length of homopolymer they are part of (only accounts for bases that were read prior to the current base)."),
-        HOMOPOLYMER(() -> homopolymerStratifier, "Stratifies bases based on the length of homopolymer, the base that the homopolymer is comprised of, and the reference base."),
+        GC_CONTENT(() -> gcContentStratifier, "The GC-content of the read."),
+        READ_ORDINALITY(() -> readOrdinalityStratifier, "The read ordinality (i.e. first or second)."),
+        READ_BASE(() -> currentReadBaseStratifier, "the base in the original reading direction."),
+        READ_DIRECTION(() -> readDirectionStratifier, "The alignment direction of the read (encoded as + or -)."),
+        PAIR_ORIENTATION(() -> readOrientationStratifier, "The read-pair's orientation (encoded as '[FR]1[FR]2')."),
+        PAIR_PROPERNESS(() -> readPairednessStratifier, "The properness of the read-pair's alignment. Looks for indications of chimerism."),
+        REFERENCE_BASE(() -> referenceBaseStratifier, "The reference base in the read's direction."),
+        PRE_DINUC(() -> preDiNucleotideStratifier, "The read base at the previous cycle, and the current reference base."),
+        POST_DINUC(() -> postDiNucleotideStratifier, "The read base at the subsequent cycle, and the current reference base."),
+        HOMOPOLYMER_LENGTH(() -> homoPolymerLengthStratifier, "The length of homopolymer the base is part of (only accounts for bases that were read prior to the current base)."),
+        HOMOPOLYMER(() -> homopolymerStratifier, "The length of homopolymer, the base that the homopolymer is comprised of, and the reference base."),
         //using a lazy initializer to enable the value of LONG_HOMOPOLYMER to be used;
-        BINNED_HOMOPOLYMER(binnedHomopolymerStratifier::get, "Stratifies bases based on the scale of homopolymer (long or short), the base that the homopolymer is comprised of, and the reference base."),
-        FLOWCELL_TILE(() -> flowCellTileStratifier, "Stratifies according to the flowcell-tile where the base was read (taken from the read name)."),
-        READ_GROUP(() -> readgroupStratifier, "Stratifies bases according to their read-group id."),
-        CYCLE(() -> baseCycleStratifier, "Stratifies bases to the machine cycle during which they were read."),
-        BINNED_CYCLE(() -> binnedReadCycleStratifier, "Stratifies bases according to the binned machine cycle in the read similar to CYCLE, but binned into 5 evenly spaced ranges across the size of the read.  This stratifier may produce confusing results when used on datasets with variable sized reads."),
-        INSERT_LENGTH(() -> insertLengthStratifier, "Stratifies bases according to the insert-size they came from (taken from the TLEN field.)"),
-        BASE_QUALITY(() -> baseQualityStratifier, "Stratifies bases according to their base quality."),
-        MAPPING_QUALITY(() -> mappingQualityStratifier, "Stratifies bases according to their read's mapping quality."),
-        MISMATCHES_IN_READ(() -> mismatchesInReadStratifier, "Stratifies bases according to the number of bases in the read that mismatch the reference excluding the current base.  This stratifier requires the NM tag."),
-        ONE_BASE_PADDED_CONTEXT(() -> oneBasePaddedContextStratifier, "Stratifies bases according the current reference base and a one base padded region from the read resulting in a 3-base context."),
-        TWO_BASE_PADDED_CONTEXT(() -> twoBasePaddedContextStratifier, "Stratifies bases according the current reference base and a two base padded region from the read resulting in a 5-base context."),
-        CONSENSUS(() -> consensusStratifier, "Stratifies bases according to whether or not duplicate reads were used to form a consensus read.  This stratifier makes use of the aD, bD, and cD tags for duplex consensus reads.  If the reads are single index consensus, only the cD tags are used."),
-        NS_IN_READ(() -> nsInReadStratifier, "Stratifies bases according to the number of Ns in the read.");
+        BINNED_HOMOPOLYMER(binnedHomopolymerStratifier::get, "The scale of homopolymer (long or short), the base that the homopolymer is comprised of, and the reference base."),
+        FLOWCELL_TILE(() -> flowCellTileStratifier, "The flowcell and tile where the base was read (taken from the read name)."),
+        READ_GROUP(() -> readgroupStratifier, "The read-group id of the read."),
+        CYCLE(() -> baseCycleStratifier, "The machine cycle during which the base was read."),
+        BINNED_CYCLE(() -> binnedReadCycleStratifier, "The binned machine cycle. Similar to CYCLE, but binned into 5 evenly spaced ranges across the size of the read.  This stratifier may produce confusing results when used on datasets with variable sized reads."),
+        SOFT_CLIPS(() -> softClipsLengthStratifier, "The number of softclipped bases the read has."),
+        INSERT_LENGTH(() -> insertLengthStratifier, "The insert-size they came from (taken from the TLEN field.)"),
+        BASE_QUALITY(() -> baseQualityStratifier, "The base quality."),
+        MAPPING_QUALITY(() -> mappingQualityStratifier, "The read's mapping quality."),
+        MISMATCHES_IN_READ(() -> mismatchesInReadStratifier, "The number of bases in the read that mismatch the reference, excluding the current base.  This stratifier requires the NM tag."),
+        ONE_BASE_PADDED_CONTEXT(() -> oneBasePaddedContextStratifier, "The current reference base and a one base padded region from the read resulting in a 3-base context."),
+        TWO_BASE_PADDED_CONTEXT(() -> twoBasePaddedContextStratifier, "The current reference base and a two base padded region from the read resulting in a 5-base context."),
+        CONSENSUS(() -> consensusStratifier, "Whether or not duplicate reads were used to form a consensus read.  This stratifier makes use of the aD, bD, and cD tags for duplex consensus reads.  If the reads are single index consensus, only the cD tags are used."),
+        NS_IN_READ(() -> nsInReadStratifier, "The number of Ns in the read."),
+        INSERTIONS_IN_READ(() -> insertionsInReadStratifier, "The number of Insertions in the read cigar."),
+        DELETIONS_IN_READ(() -> deletionsInReadStratifier, "The number of Deletions in the read cigar."),
+        INDELS_IN_READ(() -> indelsInReadStratifier, "The number of INDELs in the read cigar."),
+        INDEL_LENGTH(() -> indelLengthStratifier, "The number of bases in an indel");
 
         private final String docString;
 
@@ -705,6 +808,48 @@ public class ReadBaseStratification {
         }
     }
 
+    /**
+     * An enum to hold information about the "properness" of a read pair
+     */
+    public enum ProperPaired {
+        // Read has no supplementary alignments, is aligned to the same reference as its mate,
+        // and has the "properly aligned" flag 0x2 set.
+        PROPER,
+        // Read has no supplementary alignments, is aligned to the same reference as its mate,
+        // and has the "properly aligned" flag 0x2 UN-set.
+        IMPROPER,
+        // Read is softclipped, is aligned to the same reference as its mate, and
+        // has at least one supplementary alignment
+        CHIMERIC,
+        // Read is aligned to a different reference than its mate
+        DISCORDANT,
+        // Read or Mate are not aligned, or read has no mate and cannot be declared to be CHIMERIC.
+        UNKNOWN;
+
+        public static ProperPaired of(final SAMRecord sam) {
+
+            if (sam.getReadPairedFlag() &&
+                    !sam.getMateUnmappedFlag() && !sam.getReadUnmappedFlag() &&
+                    !sam.getMateReferenceIndex().equals(sam.getReferenceIndex())) {
+                return DISCORDANT;
+            }
+
+            if (!sam.getReadUnmappedFlag() && sam.getCigar().isClipped() && sam.hasAttribute(SAMTag.SA.toString())) {
+                return CHIMERIC;
+            }
+
+            if (sam.getReadUnmappedFlag() || sam.getReadPairedFlag() && sam.getMateUnmappedFlag()) {
+                return UNKNOWN;
+            }
+
+            if (!sam.getProperPairFlag()) {
+                return IMPROPER;
+            }
+
+            return PROPER;
+        }
+    }
+
     /**
      * An enum designed to hold a binned version of any probability-like number (between 0 and 1)
      * in quintiles
@@ -761,7 +906,7 @@ public class ReadBaseStratification {
      * An enum for holding a reads read-pair's Orientation (i.e. F1R2, F2R1, or TANDEM) indicating
      * the strand (positive or negative) that each of the two mated reads are aligned to. In connection with
      * READ_BASE and similar stratifiers this can be used to observe oxoG-type sequencing artifacts.
-     *
+     * <p>
      * Note that this is not related to FR/RF/TANDEM classification as per {@link htsjdk.samtools.SamPairUtil.PairOrientation PairOrientation}.
      */
     public enum PairOrientation {
@@ -782,10 +927,17 @@ public class ReadBaseStratification {
             final ReadOrdinality ordinality = ReadOrdinality.of(sam);
             final ReadDirection direction = ReadDirection.of(sam);
 
+            // Make sure that the read is paired
+            if (!sam.getReadPairedFlag()) {
+                return null;
+            }
+
             // if read is unmapped, read isn't mapped, or mate is unmapped, return null
             if (direction == null ||
                     sam.getReadUnmappedFlag() ||
-                    sam.getMateUnmappedFlag()) return null;
+                    sam.getMateUnmappedFlag()) {
+                return null;
+            }
 
             final boolean matePositiveStrand = !sam.getMateNegativeStrandFlag();
 
@@ -834,6 +986,22 @@ public class ReadBaseStratification {
         UNKNOWN
     }
 
+    /**
+     * Returns the number of bases associated with a specific cigar operator within a read
+     * @param samRecord The read to investigate
+     * @param operator The operator that the counted bases should be associated with
+     */
+    private static Integer stratifyCigarOperatorsInRead(final SAMRecord samRecord, final CigarOperator operator) {
+        try {
+            return samRecord.getCigar().getCigarElements().stream()
+                    .filter(ce -> ce.getOperator().equals(operator))
+                    .mapToInt(CigarElement::getLength)
+                    .sum();
+        } catch (final Exception ex) {
+            return null;
+        }
+    }
+
     /**
      * A function for stratifying a recordAndOffset and a SAMLocus into a context of bases surrounding the base in question
      * Read direction is taken into consideration (all bases are reverse-complemented if needed so that the result is in the
@@ -939,6 +1107,18 @@ public class ReadBaseStratification {
                 Math.abs(sam.getInferredInsertSize()));
     }
 
+    public static final int NOT_ALIGNED_ERROR = -1;
+
+    private static Integer stratifySoftClippedBases(final SAMRecord sam) {
+        final Cigar cigar = sam.getCigar();
+        if (cigar == null) {
+            return NOT_ALIGNED_ERROR;
+        }
+        return cigar.getCigarElements().stream()
+                .filter(e -> e.getOperator() == CigarOperator.S)
+                .mapToInt(CigarElement::getLength).sum();
+    }
+
     private static Byte stratifyBaseQuality(final RecordAndOffset recordAndOffset) {
         return recordAndOffset.getBaseQuality();
     }
@@ -950,4 +1130,83 @@ public class ReadBaseStratification {
     private static String stratifyReadGroup(final SAMRecord sam) {
         return sam.getReadGroup().getReadGroupId();
     }
-}
+
+    private static Integer stratifyIndelLength(final RecordAndOffset recordAndOffset, final SAMLocusAndReference locusInfo) {
+        // If the base is not an indel, stratify it as a length of 0
+        if (recordAndOffset.getAlignmentType() != AbstractRecordAndOffset.AlignmentType.Insertion &&
+                recordAndOffset.getAlignmentType() != AbstractRecordAndOffset.AlignmentType.Deletion) {
+            return 0;
+        }
+
+        final CigarElement cigarElement = getIndelElement(recordAndOffset);
+        if (cigarElement == null) {
+            // No CIGAR operation for given position.
+            return null;
+        }
+        if (recordAndOffset.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Insertion &&
+                cigarElement.getOperator() != CigarOperator.I) {
+            throw new IllegalStateException("Wrong CIGAR operator for the given position.");
+        }
+        if (recordAndOffset.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Deletion &&
+                cigarElement.getOperator() != CigarOperator.D) {
+            throw new IllegalStateException("Wrong CIGAR operator for the given position.");
+        }
+        return cigarElement.getLength();
+    }
+
+    public static CigarElement getIndelElement(final RecordAndOffset recordAndOffset) {
+        final SAMRecord record = recordAndOffset.getRecord();
+        final int offset = recordAndOffset.getOffset();
+
+        if (recordAndOffset.getAlignmentType() != AbstractRecordAndOffset.AlignmentType.Insertion &&
+                recordAndOffset.getAlignmentType() != AbstractRecordAndOffset.AlignmentType.Deletion) {
+            log.warn("This method is not supported for matching bases.");
+            // But could be included by handling matches the same way as insertions?
+            return null;
+        }
+
+        if (record == null) {
+            throw new IllegalArgumentException("record must not be null.");
+        }
+
+        // -1 is still a valid input for a deletion (returns the first cigar element). Everything below that is an error.
+        if (recordAndOffset.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Insertion && offset < 0) {
+            throw new IllegalArgumentException("offset must greater than zero for an insertion.");
+        }
+
+        if (recordAndOffset.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Deletion && offset < -1) {
+            throw new IllegalArgumentException("offset must greater than -1 for a deletion.");
+        }
+
+        final Cigar cigar = record.getCigar();
+        if (cigar.isEmpty())
+            return null;
+
+        if (offset == -1) {
+            return cigar.getCigarElement(0);
+        }
+
+        int readPosition = 0;
+        for (CigarElement cigarElement : cigar) {
+            if (readPosition > offset + 1) {
+                // We somehow went past the desired location
+                return null;
+            }
+
+            if (recordAndOffset.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Insertion) {
+                // If it doesn't consume bases, skip to the next
+                if (cigarElement.getOperator().consumesReadBases() && readPosition == offset) {
+                    return cigarElement;
+                }
+            }
+            else if (recordAndOffset.getAlignmentType() == AbstractRecordAndOffset.AlignmentType.Deletion) {
+                if (readPosition == offset + 1) {
+                    return cigarElement;
+                }
+            }
+
+            readPosition += cigarElement.getOperator().consumesReadBases() ? cigarElement.getLength() : 0;
+        }
+        return null;
+    }
+}
\ No newline at end of file
