@@ -24,8 +24,15 @@
 
 package picard.util;
 
+import htsjdk.utils.ValidationUtils;
+import org.apache.commons.math3.random.RandomDataGenerator;
+import picard.PicardException;
+
 import java.math.BigDecimal;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.List;
+import java.util.Random;
 
 import static java.lang.Math.log1p;
 import static java.lang.Math.pow;
@@ -36,23 +43,30 @@ import static java.lang.Math.pow;
  * @author Tim Fennell
  */
 final public class MathUtil {
-    private MathUtil(){};
+    private MathUtil() {
+    }
 
-    /** The double value closest to 1 while still being less than 1. */
+    /**
+     * The double value closest to 1 while still being less than 1.
+     */
     public static final double MAX_PROB_BELOW_ONE = 0.9999999999999999d;
 
-    /** Constant to convert between the natural base e and 4.0.  Useful for
-     * entropy calculations on DNA. */
+    /**
+     * Constant to convert between the natural base e and 4.0.  Useful for
+     * entropy calculations on DNA.
+     */
     public static final double LOG_4_BASE_E = Math.log(4.0);
 
     /**
-     *  this function mimics the behavior of log_1p but resulting in log _base 10_ of (1+x) instead of natural log of 1+x
+     * this function mimics the behavior of log_1p but resulting in log _base 10_ of (1+x) instead of natural log of 1+x
      */
-    public static double log10_1p(final double x){
-        return log1p(x)/LOG_10_MATH.log_of_base;
+    public static double log10_1p(final double x) {
+        return log1p(x) / LOG_10_MATH.log_of_base;
     }
 
-    /** Calculated the mean of an array of doubles. */
+    /**
+     * Calculated the mean of an array of doubles.
+     */
     public static double mean(final double[] in, final int start, final int stop) {
         double total = 0;
         for (int i = start; i < stop; ++i) {
@@ -62,12 +76,20 @@ final public class MathUtil {
         return total / (stop - start);
     }
 
-    /** Calculated the standard deviation of an array of doubles. */
+    public static double mean(final double[] in) {
+        return mean(in, 0, in.length);
+    }
+
+    /**
+     * Calculated the standard deviation of an array of doubles.
+     */
     public static double stddev(final double[] in, final int start, final int length) {
         return stddev(in, start, length, mean(in, start, length));
     }
 
-    /** Calculated the standard deviation of an array of doubles. */
+    /**
+     * Calculated the standard deviation of an array of doubles.
+     */
     public static double stddev(final double[] in, final int start, final int stop, final double mean) {
         double total = 0;
         for (int i = start; i < stop; ++i) {
@@ -77,11 +99,20 @@ final public class MathUtil {
         return Math.sqrt((total / (stop - start)) - (mean * mean));
     }
 
+    /**
+     * Calculated the standard deviation of an array of doubles.
+     */
+    public static double stddev(final double[] in, final double mean) {
+        return stddev(in, 0, in.length, mean);
+    }
+
     public static int compare(final int v1, final int v2) {
         return (v1 < v2 ? -1 : (v1 == v2 ? 0 : 1));
     }
 
-    /** Calculate the median of an array of doubles. Assumes that the input is sorted */
+    /**
+     * Calculate the median of an array of doubles. Assumes that the input is sorted
+     */
     public static double median(final double... in) {
         if (in.length == 0) {
             throw new IllegalArgumentException("Attempting to find the median of an empty array");
@@ -95,6 +126,7 @@ final public class MathUtil {
 
     /**
      * Obtains percentage of two Longs
+     *
      * @param numerator   dividend
      * @param denominator divisor
      * @return numerator/(double)denominator if both are non-null and denominator != 0, else returns null.
@@ -107,8 +139,8 @@ final public class MathUtil {
         }
     }
 
-    /** 
-     * Round off the value to the specified precision. 
+    /**
+     * Round off the value to the specified precision.
      */
     public static double round(final double num, final int precision) {
         BigDecimal bd = new BigDecimal(num);
@@ -118,17 +150,20 @@ final public class MathUtil {
 
     /**
      * Divide two Doubles but return 0.0 if the denominator is 0
-     * @param numerator dividend
+     *
+     * @param numerator   dividend
      * @param denominator divisor
      * @return numerator/denominator unless denominator is 0, in which case returns 0
      */
     public static double divide(final double numerator, final double denominator) {
-        return Math.abs(0.0 - denominator) >  0.000001
+        return Math.abs(0.0 - denominator) > 0.000001
                 ? numerator / denominator
                 : 0.0;
     }
 
-    /** Returns the largest value stored in the array. */
+    /**
+     * Returns the largest value stored in the array.
+     */
     public static double max(final double[] nums) {
         return nums[indexOfMax(nums)];
     }
@@ -139,7 +174,7 @@ final public class MathUtil {
      */
     public static int indexOfMax(final double[] nums) {
         double max = nums[0];
-        int index  = 0;
+        int index = 0;
         for (int i = 1; i < nums.length; ++i) {
             if (nums[i] > max) {
                 max = nums[i];
@@ -150,7 +185,9 @@ final public class MathUtil {
         return index;
     }
 
-    /** Returns the largest value stored in the array. */
+    /**
+     * Returns the largest value stored in the array.
+     */
     public static long max(final long[] nums) {
         return nums[indexOfMax(nums)];
     }
@@ -161,7 +198,7 @@ final public class MathUtil {
      */
     public static int indexOfMax(final long[] nums) {
         long max = nums[0];
-        int index  = 0;
+        int index = 0;
         for (int i = 1; i < nums.length; ++i) {
             if (nums[i] > max) {
                 max = nums[i];
@@ -172,44 +209,55 @@ final public class MathUtil {
         return index;
     }
 
-    /** Returns the smallest value stored in the array. */
+    /**
+     * Returns the smallest value stored in the array.
+     */
     public static double min(final double[] nums) {
         return nums[indexOfMin(nums)];
     }
 
-    /** Returns the smallest value stored in the array. */
+    /**
+     * Returns the smallest value stored in the array.
+     */
     public static int min(final int[] nums) {
         return nums[indexOfMin(nums)];
     }
-    
-    /** Returns the smallest value stored in the array. */
+
+    /**
+     * Returns the smallest value stored in the array.
+     */
     public static short min(final short[] nums) {
         short min = nums[0];
         for (int i = 1; i < nums.length; ++i) {
-            if (nums[i] < min) min = nums[i];
+            if (nums[i] < min) {
+                min = nums[i];
+            }
         }
 
         return min;
     }
 
-    /** Returns the smallest value stored in the array. */
+    /**
+     * Returns the smallest value stored in the array.
+     */
     public static byte min(final byte[] nums) {
         byte min = nums[0];
         for (int i = 1; i < nums.length; ++i) {
-            if (nums[i] < min) min = nums[i];
+            if (nums[i] < min) {
+                min = nums[i];
+            }
         }
 
         return min;
     }
 
-
     /**
      * Returns the index of the smallest element in the array.  If there are multiple equal minima then
      * the earliest one in the array is returned.
      */
     public static int indexOfMin(final int[] nums) {
         long min = nums[0];
-        int index  = 0;
+        int index = 0;
         for (int i = 1; i < nums.length; ++i) {
             if (nums[i] < min) {
                 min = nums[i];
@@ -226,7 +274,7 @@ final public class MathUtil {
      */
     public static int indexOfMin(final double[] nums) {
         double min = nums[0];
-        int index  = 0;
+        int index = 0;
         for (int i = 1; i < nums.length; ++i) {
             if (nums[i] < min) {
                 min = nums[i];
@@ -236,17 +284,24 @@ final public class MathUtil {
 
         return index;
     }
-    /** Mimic's R's seq() function to produce a sequence of equally spaced numbers. */
+
+    /**
+     * Mimic's R's seq() function to produce a sequence of equally spaced numbers.
+     */
     public static double[] seq(final double from, final double to, final double by) {
-        if (from < to && by <= 0) return new double[0];
-        if (from > to && by >= 0) return new double[0];
+        if (from < to && by <= 0) {
+            return new double[0];
+        }
+        if (from > to && by >= 0) {
+            return new double[0];
+        }
         final int values = 1 + (int) Math.floor((to - from) / by);
         final double[] results = new double[values];
 
         BigDecimal value = new BigDecimal(from);
         BigDecimal increment = new BigDecimal(by);
 
-        for (int i=0; i<values; ++i) {
+        for (int i = 0; i < values; ++i) {
             results[i] = value.doubleValue();
             value = value.add(increment);
         }
@@ -254,10 +309,14 @@ final public class MathUtil {
         return results;
     }
 
-    /** "Promotes" an int[] into a double array with the same values (or as close as precision allows). */
+    /**
+     * "Promotes" an int[] into a double array with the same values (or as close as precision allows).
+     */
     public static double[] promote(final int[] is) {
         final double[] ds = new double[is.length];
-        for (int i = 0; i < is.length; ++i) ds[i] = is[i];
+        for (int i = 0; i < is.length; ++i) {
+            ds[i] = is[i];
+        }
         return ds;
     }
 
@@ -266,20 +325,56 @@ final public class MathUtil {
         return pNormalizeLogProbability(likelihoods);
     }
 
+    /**
+     * Takes a complete set of mutually exclusive logLikelihoods and converts them to probabilities
+     * with no rescaling. Will throw if underflow is detected (if all the likelihoods are less than -300)
+     */
+    public static double[] getProbabilityFromLog(final double[] lLikelihood) {
+
+        final double maxLikelihood = max(lLikelihood);
+        if (maxLikelihood < -300) {
+            throw new PicardException(String.format("Underflow detected, needed to calculate pow(10,x) with x=%f.", maxLikelihood));
+        }
+
+        final double[] tmp = new double[lLikelihood.length];
+        for (int i = 0; i < lLikelihood.length; ++i) {
+            tmp[i] = pow(10, lLikelihood[i]);
+        }
+
+        return tmp;
+    }
+
+    /**
+     * Takes a complete set of mutually exclusive Likelihoods and converts them to logLikelihoods.
+     */
+    public static double[] getLogFromProbability(final double[] likelihood) {
+
+        final double[] tmp = new double[likelihood.length];
+        for (int i = 0; i < likelihood.length; ++i) {
+            tmp[i] = Math.log10(likelihood[i]);
+        }
+
+        return tmp;
+    }
+
     /**
      * Takes a complete set of mutually exclusive logPosteriors and converts them to probabilities
      * that sum to 1 with as much fidelity as possible.  Limits probabilities to be in the space:
      * 0.9999999999999999 >= p >= (1-0.9999999999999999)/(lPosteriors.length-1)
      */
     public static double[] pNormalizeLogProbability(final double[] lPosterior) {
+
+        final double[] tmp = new double[lPosterior.length];
+        if (lPosterior.length == 0) {
+            return tmp;
+        }
+
         // Note: bumping all the LLs so that the biggest is 300 ensures that we have the
         // widest range possible when unlogging them before one of them underflows. 10^300 is
         // near the maximum before you hit positive infinity.
-
         final double maxLikelihood = max(lPosterior);
         final double bump = 300 - maxLikelihood;
 
-        final double[] tmp = new double[lPosterior.length];
         double total = 0;
         for (int i = 0; i < lPosterior.length; ++i) {
             tmp[i] = pow(10, lPosterior[i] + bump);
@@ -291,20 +386,29 @@ final public class MathUtil {
 
         for (int i = 0; i < lPosterior.length; ++i) {
             tmp[i] /= total;
-            if (tmp[i] > maxP) tmp[i] = maxP;
-            else if (tmp[i] < minP) tmp[i] = minP;
+            if (tmp[i] > maxP) {
+                tmp[i] = maxP;
+            } else if (tmp[i] < minP) {
+                tmp[i] = minP;
+            }
         }
 
         return tmp;
     }
 
-    /** Calculates the ratio of two arrays of the same length. */
+    /**
+     * Calculates the ratio of two arrays of the same length.
+     */
     public static double[] divide(final double[] numerators, final double[] denominators) {
-        if (numerators.length != denominators.length) throw new IllegalArgumentException("Arrays must be of same length.");
+        if (numerators.length != denominators.length) {
+            throw new IllegalArgumentException("Arrays must be of same length.");
+        }
 
         final int len = numerators.length;
         final double[] result = new double[len];
-        for (int i = 0; i < len; ++i) result[i] = numerators[i] / denominators[i];
+        for (int i = 0; i < len; ++i) {
+            result[i] = numerators[i] / denominators[i];
+        }
         return result;
     }
 
@@ -323,49 +427,94 @@ final public class MathUtil {
 
         for (int i = 0; i < pPosterior.length; ++i) {
             tmp[i] = pPosterior[i] / total;
-            if (tmp[i] > maxP) tmp[i] = maxP;
-            else if (tmp[i] < minP) tmp[i] = minP;
+            if (tmp[i] > maxP) {
+                tmp[i] = maxP;
+            } else if (tmp[i] < minP) {
+                tmp[i] = minP;
+            }
         }
 
         return tmp;
     }
 
-    /** Calculates the product of two arrays of the same length. */
+    /**
+     * Calculates the product of two arrays of the same length.
+     */
     public static double[] multiply(final double[] lhs, final double[] rhs) {
-        if (lhs.length != rhs.length) throw new IllegalArgumentException("Arrays must be of same length.");
+        if (lhs.length != rhs.length) {
+            throw new IllegalArgumentException("Arrays must be of same length.");
+        }
 
         final int len = lhs.length;
         final double[] result = new double[len];
-        for (int i = 0; i < len; ++i) result[i] = lhs[i] * rhs[i];
+        for (int i = 0; i < len; ++i) {
+            result[i] = lhs[i] * rhs[i];
+        }
         return result;
     }
 
-    /** calculates the sum of the arrays as a third array. */
+    /**
+     * calculates the sum of the arrays as a third array.
+     */
     public static double[] sum(final double[] lhs, final double[] rhs) {
-        if (lhs.length != rhs.length) throw new IllegalArgumentException("Arrays must be of same length.");
+        if (lhs.length != rhs.length) {
+            throw new IllegalArgumentException("Arrays must be of same length.");
+        }
 
         final int len = lhs.length;
-        final double [] result = new double[len];
-        for (int i = 0; i < len; ++i) result[i] = lhs[i] + rhs[i];
+        final double[] result = new double[len];
+        for (int i = 0; i < len; ++i) {
+            result[i] = lhs[i] + rhs[i];
+        }
         return result;
     }
 
-    /** Returns the sum of the elements in the array. */
+    /**
+     * calculates the sum of an array and a double.
+     */
+    public static double[] sum(final double[] lhs, final double rhs) {
+
+        final int len = lhs.length;
+        final double[] result = new double[len];
+        for (int i = 0; i < len; ++i) {
+            result[i] = lhs[i] + rhs;
+        }
+        return result;
+    }
+
+    /**
+     * Returns the sum of the elements in the array.
+     */
     public static double sum(final double[] arr) {
         double result = 0;
-        for (final double next : arr) result += next;
+        for (final double next : arr) {
+            result += next;
+        }
         return result;
     }
 
-    /** Returns the sum of the elements in the array starting with start and ending before stop. */
+    /**
+     * Returns the sum of the elements in the array starting with start and ending before stop.
+     */
     public static long sum(final long[] arr, final int start, final int stop) {
         long result = 0;
-        for (int i=start; i<stop; ++i) {
+        for (int i = start; i < stop; ++i) {
             result += arr[i];
         }
         return result;
     }
 
+    /**
+     * returns a long array containing the rounded values of the input array
+     */
+    public static long[] round(final double... input) {
+        final long[] out = new long[input.length];
+        for (int i = 0; i < input.length; i++) {
+            out[i] = Math.round(input[i]);
+        }
+        return out;
+    }
+
     public static final LogMath LOG_2_MATH = new LogMath(2);
     public static final LogMath NATURAL_LOG_MATH = new LogMath(Math.exp(1)) {
         @Override
@@ -380,8 +529,71 @@ final public class MathUtil {
             return Math.log10(nonLogValue);
         }
     };
-    
-    /** 
+
+    /**
+     * Calculate the KL divergence from measured to distribution
+     * // https://en.wikipedia.org/wiki/Kullback%E2%80%93Leibler_divergence
+     */
+    public static double klDivergance(double[] measured, double[] distribution) {
+        ValidationUtils.validateArg(measured.length == distribution.length, () -> "length of measuered and distribution differ. Found " + measured.length + " and " + distribution.length + ".");
+
+
+        final double[] normalizedMeasured = pNormalizeVector(measured);
+        final double[] normalizedDistribution = pNormalizeVector(distribution);
+
+        return -sum(multiply(normalizedMeasured, NATURAL_LOG_MATH.getLogValue(divide(normalizedDistribution, normalizedMeasured))));
+    }
+
+    /**
+     * permute the input array randomly (using a RandomDataGenerator)
+     *
+     * @param array input array
+     * @param rdg   a RandomDataGenerator for drawing a permutation from
+     * @return a newly allocated array with a permuted version of the original data.
+     */
+    public static double[] permute(double[] array, RandomDataGenerator rdg) {
+
+        final int n = array.length;
+        final double[] retVal = new double[n];
+        final int[] randomPermutation = rdg.nextPermutation(n, n);
+        for (int i = 0; i < n; i++) {
+            retVal[i] = array[randomPermutation[i]];
+        }
+        return retVal;
+    }
+
+    /**
+     * A small utility function to choose n random elements (un-shuffled) from a list
+     *
+     * @param list   A list of elements
+     * @param n      a number of elements requested from list
+     * @param random a Random object to use for subsetting
+     * @return a list of n randomly chosen (but in the original order) elements from list.
+     * If the list has less than n elements it is returned in its entirety.
+     */
+    public static <T> List<T> randomSublist(final List<T> list, final int n, final Random random) {
+        int availableElements = list.size();
+        if (availableElements <= n) {
+            return list;
+        }
+
+        int stillNeeded = n;
+        final List<T> shortList = new ArrayList<>(n);
+        for (final T aList : list) {
+            if (random.nextDouble() < stillNeeded / (double) availableElements) {
+                shortList.add(aList);
+                stillNeeded--;
+            }
+            if (stillNeeded == 0) {
+                break; // fast out if do not need more elements
+            }
+            availableElements--;
+        }
+
+        return shortList;
+    }
+
+    /**
      * A collection of common math operations that work with log values. To use it, pass values from log space, the operation will be
      * computed in non-log space, and a value in log space will be returned.
      */
@@ -395,20 +607,26 @@ final public class MathUtil {
 
         private LogMath(final double base) {
             this.base = base;
-            this.log_of_base=Math.log(base);
+            this.log_of_base = Math.log(base);
         }
 
-        /** Returns the decimal representation of the provided log values. */
+        /**
+         * Returns the decimal representation of the provided log values.
+         */
         public double getNonLogValue(final double logValue) {
             return Math.pow(base, logValue);
         }
 
-        /** Returns the log-representation of the provided decimal value. */
+        /**
+         * Returns the log-representation of the provided decimal value.
+         */
         public double getLogValue(final double nonLogValue) {
             return Math.log(nonLogValue) / log_of_base;
         }
 
-        /** Returns the log-representation of the provided decimal array. */
+        /**
+         * Returns the log-representation of the provided decimal array.
+         */
         public double[] getLogValue(final double[] nonLogArray) {
             final double[] logArray = new double[nonLogArray.length];
             for (int i = 0; i < nonLogArray.length; i++) {
@@ -417,12 +635,16 @@ final public class MathUtil {
             return logArray;
         }
 
-        /** Computes the mean of the provided log values. */
+        /**
+         * Computes the mean of the provided log values.
+         */
         public double mean(final double... logValues) {
             return sum(logValues) - getLogValue(logValues.length);
         }
-        
-        /** Computes the sum of the provided log values. */
+
+        /**
+         * Computes the sum of the provided log values.
+         */
         public double sum(final double... logValues) {
             // Avoid overflow via scaling.
             final double scalingFactor = max(logValues);
@@ -432,12 +654,13 @@ final public class MathUtil {
             }
             return getLogValue(simpleAdditionResult) + scalingFactor;
         }
- 
-        /** Computes the sum of the provided log values. */
+
+        /**
+         * Computes the sum of the provided log values.
+         */
         public double product(final double... logValues) {
             return MathUtil.sum(logValues);
         }
 
-
     }
 }
