@@ -46,22 +46,24 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public String getCommandLineProgramName() {
         return CollectAlignmentSummaryMetrics.class.getSimpleName();
     }
-    
+
     @Test
     public void test() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
         outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
         final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
 
         Assert.assertEquals(output.getMetrics().size(), 3);
         for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
@@ -121,60 +123,62 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void testBisulfite() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_bisulfite_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
         outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
                 "IS_BISULFITE_SEQUENCED=true"
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final NumberFormat format =  NumberFormat.getInstance();
+        final NumberFormat format = NumberFormat.getInstance();
         format.setMaximumFractionDigits(4);
 
         final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
 
         for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
             Assert.assertEquals(metrics.MEAN_READ_LENGTH, 101.0);
             switch (metrics.CATEGORY) {
-            case FIRST_OF_PAIR:
-                // 19 no-calls, one potentially methylated base, one mismatch at a potentially methylated base
-                Assert.assertEquals(metrics.TOTAL_READS, 1);
-                Assert.assertEquals(metrics.PF_READS, 1);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_BASES, 101);
-                Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 21.0);
-                Assert.assertEquals(metrics.PF_ALIGNED_BASES, 101);
-                Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.212121 /*21D/99D*/);
-                Assert.assertEquals(metrics.BAD_CYCLES, 21);
-                Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(21/(double)99));
-                break;
-            case SECOND_OF_PAIR:
-                // Three no-calls, two potentially methylated bases
-                Assert.assertEquals(metrics.TOTAL_READS, 1);
-                Assert.assertEquals(metrics.PF_READS, 1);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_BASES, 101);
-                Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 4.0);
-                Assert.assertEquals(metrics.PF_ALIGNED_BASES, 101);
-                Assert.assertEquals(metrics.PF_MISMATCH_RATE, /*4D/99D*/0.040404);
-                Assert.assertEquals(metrics.BAD_CYCLES, 4);
-                Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(4/(double)99));
-                break;
-            case PAIR:
-                Assert.assertEquals(metrics.TOTAL_READS, 2);
-                Assert.assertEquals(metrics.PF_READS, 2);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_BASES, 202);
-                Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 12.5D);
-                Assert.assertEquals(metrics.PF_ALIGNED_BASES, 202);
-                Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.126263);// 25D/198D
-                Assert.assertEquals(metrics.BAD_CYCLES, 25);
-                Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(25/(double)198));
-                break;
-            case UNPAIRED:
-            default:
-                Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
+                case FIRST_OF_PAIR:
+                    // 19 no-calls, one potentially methylated base, one mismatch at a potentially methylated base
+                    Assert.assertEquals(metrics.TOTAL_READS, 1);
+                    Assert.assertEquals(metrics.PF_READS, 1);
+                    Assert.assertEquals(metrics.PF_HQ_ALIGNED_BASES, 101);
+                    Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 21.0);
+                    Assert.assertEquals(metrics.PF_ALIGNED_BASES, 101);
+                    Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.212121 /*21D/99D*/);
+                    Assert.assertEquals(metrics.BAD_CYCLES, 21);
+                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(21 / (double) 99));
+                    break;
+                case SECOND_OF_PAIR:
+                    // Three no-calls, two potentially methylated bases
+                    Assert.assertEquals(metrics.TOTAL_READS, 1);
+                    Assert.assertEquals(metrics.PF_READS, 1);
+                    Assert.assertEquals(metrics.PF_HQ_ALIGNED_BASES, 101);
+                    Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 4.0);
+                    Assert.assertEquals(metrics.PF_ALIGNED_BASES, 101);
+                    Assert.assertEquals(metrics.PF_MISMATCH_RATE, /*4D/99D*/0.040404);
+                    Assert.assertEquals(metrics.BAD_CYCLES, 4);
+                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(4 / (double) 99));
+                    break;
+                case PAIR:
+                    Assert.assertEquals(metrics.TOTAL_READS, 2);
+                    Assert.assertEquals(metrics.PF_READS, 2);
+                    Assert.assertEquals(metrics.PF_HQ_ALIGNED_BASES, 202);
+                    Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 12.5D);
+                    Assert.assertEquals(metrics.PF_ALIGNED_BASES, 202);
+                    Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.126263);// 25D/198D
+                    Assert.assertEquals(metrics.BAD_CYCLES, 25);
+                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(25 / (double) 198));
+                    break;
+                case UNPAIRED:
+                default:
+                    Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
             }
         }
     }
@@ -183,21 +187,23 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void testBisulfiteButNot() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_bisulfite_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
         outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
                 "IS_BISULFITE_SEQUENCED=false"
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final NumberFormat format =  NumberFormat.getInstance();
+        final NumberFormat format = NumberFormat.getInstance();
         format.setMaximumFractionDigits(4);
 
         final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
 
         for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
             Assert.assertEquals(metrics.MEAN_READ_LENGTH, 101.0);
@@ -211,7 +217,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                     Assert.assertEquals(metrics.PF_ALIGNED_BASES, 101);
                     Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.227723 /*23D/101D*/);
                     Assert.assertEquals(metrics.BAD_CYCLES, 23);
-                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(23/(double)101));
+                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(23 / (double) 101));
                     break;
                 case SECOND_OF_PAIR:
                     // Three no-calls, two potentially methylated bases
@@ -222,7 +228,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                     Assert.assertEquals(metrics.PF_ALIGNED_BASES, 101);
                     Assert.assertEquals(metrics.PF_MISMATCH_RATE, /*6D/101D*/0.059406);
                     Assert.assertEquals(metrics.BAD_CYCLES, 6);
-                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(6/(double)101));
+                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(6 / (double) 101));
                     break;
                 case PAIR:
                     Assert.assertEquals(metrics.TOTAL_READS, 2);
@@ -232,7 +238,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                     Assert.assertEquals(metrics.PF_ALIGNED_BASES, 202);
                     Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.143564);// 29D/202D
                     Assert.assertEquals(metrics.BAD_CYCLES, 29);
-                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(29/(double)202));
+                    Assert.assertEquals(format.format(metrics.PF_HQ_ERROR_RATE), format.format(29 / (double) 202));
                     break;
                 case UNPAIRED:
                 default:
@@ -244,100 +250,23 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     @Test
     public void testNoReference() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test.sam");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
         outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
+                "COLLECT_ALIGNMENT_INFORMATION=false"
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
         final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
 
         for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
             Assert.assertEquals(metrics.MEAN_READ_LENGTH, 101.0);
             switch (metrics.CATEGORY) {
-            case FIRST_OF_PAIR:
-                Assert.assertEquals(metrics.TOTAL_READS, 9);
-                Assert.assertEquals(metrics.PF_READS, 7);
-                Assert.assertEquals(metrics.PF_NOISE_READS, 1);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_READS, 0);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_Q20_BASES, 0);
-                Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 0.0);
-                Assert.assertEquals(metrics.PF_ALIGNED_BASES, 0);
-                Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.0);
-                Assert.assertEquals(metrics.BAD_CYCLES, 19);
-                break;
-            case SECOND_OF_PAIR:
-                Assert.assertEquals(metrics.TOTAL_READS, 9);
-                Assert.assertEquals(metrics.PF_READS, 9);
-                Assert.assertEquals(metrics.PF_NOISE_READS, 1);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_READS, 0);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_Q20_BASES, 0);
-                Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 0.0);
-                Assert.assertEquals(metrics.PF_ALIGNED_BASES, 0);
-                Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.0);
-                Assert.assertEquals(metrics.BAD_CYCLES, 3);
-                break;
-            case PAIR:
-                Assert.assertEquals(metrics.TOTAL_READS, 18);
-                Assert.assertEquals(metrics.PF_READS, 16);
-                Assert.assertEquals(metrics.PF_NOISE_READS, 2);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_READS, 0);
-                Assert.assertEquals(metrics.PF_HQ_ALIGNED_Q20_BASES, 0);
-                Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 0.0);
-                Assert.assertEquals(metrics.PF_ALIGNED_BASES, 0);
-                Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.0);
-                Assert.assertEquals(metrics.BAD_CYCLES, 22);
-                break;
-            case UNPAIRED:
-            default:
-                Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
-            }
-        }
-    }
-
-    @Test
-    public void testZeroLengthReads() throws IOException {
-        final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test2.sam");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
-                "OUTPUT=" + outfile.getAbsolutePath(),
-        };
-        Assert.assertEquals(runPicardCommandLine(args), 0);
-
-        final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
-        for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
-            // test that it doesn't blow up
-        }
-    }
-
-    @Test
-    public void testMultipleLevelsOfMetrics() throws IOException {
-        final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_multiple.sam");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
-                "OUTPUT=" + outfile.getAbsolutePath(),
-                "METRIC_ACCUMULATION_LEVEL=ALL_READS",
-                "METRIC_ACCUMULATION_LEVEL=SAMPLE",
-                "METRIC_ACCUMULATION_LEVEL=LIBRARY",
-                "METRIC_ACCUMULATION_LEVEL=READ_GROUP",
-        };
-        Assert.assertEquals(runPicardCommandLine(args), 0);
-
-        final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
-
-        for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
-            Assert.assertEquals(metrics.MEAN_READ_LENGTH, 101.0);
-            if (metrics.SAMPLE == null) {
-                switch (metrics.CATEGORY) {
                 case FIRST_OF_PAIR:
                     Assert.assertEquals(metrics.TOTAL_READS, 9);
                     Assert.assertEquals(metrics.PF_READS, 7);
@@ -374,9 +303,94 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                 case UNPAIRED:
                 default:
                     Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
-                }
             }
-            else if (metrics.SAMPLE.equals("Ma")) {
+        }
+    }
+
+    @Test
+    public void testZeroLengthReads() throws IOException {
+        final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test2.sam");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
+        outfile.deleteOnExit();
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
+                "OUTPUT=" + outfile.getAbsolutePath(),
+                "COLLECT_ALIGNMENT_INFORMATION=false"
+        };
+        Assert.assertEquals(runPicardCommandLine(args), 0);
+
+        final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
+            // test that it doesn't blow up
+        }
+    }
+
+    @Test
+    public void testMultipleLevelsOfMetrics() throws IOException {
+        final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_multiple.sam");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
+        outfile.deleteOnExit();
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
+                "OUTPUT=" + outfile.getAbsolutePath(),
+                "METRIC_ACCUMULATION_LEVEL=ALL_READS",
+                "METRIC_ACCUMULATION_LEVEL=SAMPLE",
+                "METRIC_ACCUMULATION_LEVEL=LIBRARY",
+                "METRIC_ACCUMULATION_LEVEL=READ_GROUP",
+                "COLLECT_ALIGNMENT_INFORMATION=false"
+        };
+        Assert.assertEquals(runPicardCommandLine(args), 0);
+
+        final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+
+        for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
+            Assert.assertEquals(metrics.MEAN_READ_LENGTH, 101.0);
+            if (metrics.SAMPLE == null) {
+                switch (metrics.CATEGORY) {
+                    case FIRST_OF_PAIR:
+                        Assert.assertEquals(metrics.TOTAL_READS, 9);
+                        Assert.assertEquals(metrics.PF_READS, 7);
+                        Assert.assertEquals(metrics.PF_NOISE_READS, 1);
+                        Assert.assertEquals(metrics.PF_HQ_ALIGNED_READS, 0);
+                        Assert.assertEquals(metrics.PF_HQ_ALIGNED_Q20_BASES, 0);
+                        Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 0.0);
+                        Assert.assertEquals(metrics.PF_ALIGNED_BASES, 0);
+                        Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.0);
+                        Assert.assertEquals(metrics.BAD_CYCLES, 19);
+                        break;
+                    case SECOND_OF_PAIR:
+                        Assert.assertEquals(metrics.TOTAL_READS, 9);
+                        Assert.assertEquals(metrics.PF_READS, 9);
+                        Assert.assertEquals(metrics.PF_NOISE_READS, 1);
+                        Assert.assertEquals(metrics.PF_HQ_ALIGNED_READS, 0);
+                        Assert.assertEquals(metrics.PF_HQ_ALIGNED_Q20_BASES, 0);
+                        Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 0.0);
+                        Assert.assertEquals(metrics.PF_ALIGNED_BASES, 0);
+                        Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.0);
+                        Assert.assertEquals(metrics.BAD_CYCLES, 3);
+                        break;
+                    case PAIR:
+                        Assert.assertEquals(metrics.TOTAL_READS, 18);
+                        Assert.assertEquals(metrics.PF_READS, 16);
+                        Assert.assertEquals(metrics.PF_NOISE_READS, 2);
+                        Assert.assertEquals(metrics.PF_HQ_ALIGNED_READS, 0);
+                        Assert.assertEquals(metrics.PF_HQ_ALIGNED_Q20_BASES, 0);
+                        Assert.assertEquals(metrics.PF_HQ_MEDIAN_MISMATCHES, 0.0);
+                        Assert.assertEquals(metrics.PF_ALIGNED_BASES, 0);
+                        Assert.assertEquals(metrics.PF_MISMATCH_RATE, 0.0);
+                        Assert.assertEquals(metrics.BAD_CYCLES, 22);
+                        break;
+                    case UNPAIRED:
+                    default:
+                        Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
+                }
+            } else if (metrics.SAMPLE.equals("Ma")) {
                 // There's only one library and one read group for this sample so the metrics for
                 // every level should be identical
                 switch (metrics.CATEGORY) {
@@ -417,8 +431,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                     default:
                         Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
                 }
-            }
-            else if (metrics.SAMPLE.equals("Pa")) {
+            } else if (metrics.SAMPLE.equals("Pa")) {
                 // Two libraries and three read groups for this sample
                 if (metrics.LIBRARY == null) {
                     switch (metrics.CATEGORY) {
@@ -459,8 +472,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                         default:
                             Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
                     }
-                }
-                else if (metrics.LIBRARY.equals("lib1")) {
+                } else if (metrics.LIBRARY.equals("lib1")) {
                     // Only one read group in this library so library and RG metrics should be identical
                     switch (metrics.CATEGORY) {
                         case FIRST_OF_PAIR:
@@ -486,8 +498,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                             Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
                     }
 
-                }
-                else if (metrics.LIBRARY.equals("lib2")) {
+                } else if (metrics.LIBRARY.equals("lib2")) {
                     if (metrics.READ_GROUP == null) {
                         switch (metrics.CATEGORY) {
                             case FIRST_OF_PAIR:
@@ -512,8 +523,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                             default:
                                 Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
                         }
-                    }
-                    else if (metrics.READ_GROUP.equals("i")) {
+                    } else if (metrics.READ_GROUP.equals("i")) {
                         switch (metrics.CATEGORY) {
                             case FIRST_OF_PAIR:
                                 Assert.assertEquals(metrics.TOTAL_READS, 1);
@@ -537,8 +547,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                             default:
                                 Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
                         }
-                    }
-                    else if (metrics.READ_GROUP.equals("i2")) {
+                    } else if (metrics.READ_GROUP.equals("i2")) {
                         switch (metrics.CATEGORY) {
                             case FIRST_OF_PAIR:
                                 Assert.assertEquals(metrics.TOTAL_READS, 1);
@@ -562,17 +571,14 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
                             default:
                                 Assert.fail("Data does not contain this category: " + metrics.CATEGORY);
                         }
-                    }
-                    else {
+                    } else {
                         Assert.fail("Data does not contain this read group: " + metrics.READ_GROUP);
                     }
 
-                }
-                else {
+                } else {
                     Assert.fail("Data does not contain this library: " + metrics.LIBRARY);
                 }
-            }
-            else {
+            } else {
                 Assert.fail("Data does not contain this sample: " + metrics.SAMPLE);
             }
         }
@@ -582,10 +588,10 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void testChimeras() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_chimeras.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile   = File.createTempFile("alignmentMetrics", ".txt");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
         outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "MAX_INSERT_SIZE=20",
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
@@ -593,7 +599,9 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
         final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
 
         for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
             if (metrics.CATEGORY == AlignmentSummaryMetrics.Category.FIRST_OF_PAIR) {
@@ -604,4 +612,34 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
             }
         }
     }
+
+    @Test
+    public void testAdapterReads() throws IOException {
+        final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_adapter_reads.sam");
+        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
+        outfile.deleteOnExit();
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
+                "OUTPUT=" + outfile.getAbsolutePath(),
+                "MAX_INSERT_SIZE=200",
+                "REFERENCE_SEQUENCE=" + CHR_M_REFERENCE.getAbsolutePath(),
+        };
+        Assert.assertEquals(runPicardCommandLine(args), 0);
+
+        final MetricsFile<AlignmentSummaryMetrics, Comparable<?>> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+
+        for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
+            if (metrics.CATEGORY == AlignmentSummaryMetrics.Category.FIRST_OF_PAIR) {
+                TestNGUtil.compareDoubleWithAccuracy(metrics.PCT_ADAPTER, 0D, 0.0001);
+                TestNGUtil.compareDoubleWithAccuracy(metrics.PCT_PF_READS_ALIGNED, 1D, 0.0001);
+            }
+            if (metrics.CATEGORY == AlignmentSummaryMetrics.Category.SECOND_OF_PAIR) {
+                TestNGUtil.compareDoubleWithAccuracy(metrics.PCT_ADAPTER, 1D, 0.0001);
+                TestNGUtil.compareDoubleWithAccuracy(metrics.PCT_PF_READS_ALIGNED, 0D, 0.0001);
+            }
+        }
+    }
 }
