@@ -24,49 +24,44 @@
 
 package picard.analysis;
 
-import htsjdk.samtools.SAMException;
-import htsjdk.samtools.SAMFileHeader;
-import htsjdk.samtools.SAMFileWriter;
-import htsjdk.samtools.SAMFileWriterFactory;
-import htsjdk.samtools.SAMReadGroupRecord;
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMRecordSetBuilder;
+import htsjdk.samtools.*;
 import htsjdk.samtools.metrics.MetricsFile;
 import htsjdk.samtools.util.Histogram;
 import htsjdk.variant.utils.SAMSequenceDictionaryExtractor;
-import htsjdk.variant.vcf.VCF3Codec;
 import org.testng.Assert;
 import org.testng.annotations.BeforeTest;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 import picard.cmdline.CommandLineProgramTest;
 import picard.sam.SortSam;
+import picard.util.TestNGUtil;
 import picard.vcf.VcfTestUtils;
 
-import java.io.*;
+import java.io.File;
+import java.io.FileReader;
+import java.io.IOException;
 import java.util.Random;
 
 /**
  * Tests for methods in CollectWgsMetrics
  *
- *
  * @author Kylee Bergin
  */
 
 public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
-    private final static File REF_DICT_DIR = new File("testdata/picard/sam/CollectGcBiasMetrics/");
-    private final static File TEST_DIR = new File("testdata/picard/sam/");
+    private static final File TEST_DIR = new File("testdata/picard/sam/");
+    private static final File REF_DICT_DIR = new File(TEST_DIR, "CollectGcBiasMetrics/");
     private final File referenceDict = new File(REF_DICT_DIR, "MSmallHeader.dict");
     private File tempSamFile;
     private File outfile;
 
-    private final static int READ_PAIR_DISTANCE = 99;
-    private final static String SAMPLE = "TestSample1";
-    private final static String READ_GROUP_ID = "TestReadGroup1";
-    private final static String PLATFORM = "ILLUMINA";
-    private final static String LIBRARY = "TestLibrary1";
-    private final static int NUM_READS = 40000;
+    private static final int READ_PAIR_DISTANCE = 99;
+    private static final String SAMPLE = "TestSample1";
+    private static final String READ_GROUP_ID = "TestReadGroup1";
+    private static final String PLATFORM = "ILLUMINA";
+    private static final String LIBRARY = "TestLibrary1";
+    private static final int NUM_READS = 40000;
 
     public String getCommandLineProgramName() {
         return CollectWgsMetrics.class.getSimpleName();
@@ -74,9 +69,9 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
     @DataProvider(name = "wgsDataProvider")
     public Object[][] wgsDataProvider() {
-        final String referenceFile = "testdata/picard/quality/chrM.reference.fasta";
+        final String referenceFile = CHR_M_REFERENCE.getAbsolutePath();
 
-        return new Object[][] {
+        return new Object[][]{
                 {tempSamFile, outfile, referenceFile, "false"},
                 {tempSamFile, outfile, referenceFile, "true"},
         };
@@ -84,12 +79,12 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
     @Test(dataProvider = "wgsDataProvider")
     public void testMetricsFromWGS(final File input, final File outfile, final String referenceFile,
-            final String useFastAlgorithm) throws IOException {
+                                   final String useFastAlgorithm) throws IOException {
         outfile.deleteOnExit();
         final int sampleSize = 1000;
 
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + referenceFile,
                 "SAMPLE_SIZE=" + sampleSize,
@@ -97,16 +92,18 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final MetricsFile<CollectWgsMetrics.WgsMetrics, Comparable<?>> output = new MetricsFile<CollectWgsMetrics.WgsMetrics, Comparable<?>>();
-        output.read(new FileReader(outfile));
-
-        for (final CollectWgsMetrics.WgsMetrics metrics : output.getMetrics()) {
+        final MetricsFile<WgsMetrics, Comparable<?>> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        for (final WgsMetrics metrics : output.getMetrics()) {
             Assert.assertEquals(metrics.MEAN_COVERAGE, 13.985155, .02);
             Assert.assertEquals(metrics.PCT_EXC_OVERLAP, 0.0);  // 52 of 606 bases
             Assert.assertEquals(metrics.PCT_EXC_BASEQ, 0.399906, .02);    // 114 of 606 bases
             Assert.assertEquals(metrics.PCT_EXC_DUPE, 0.0);    // 202 of 606 bases
             Assert.assertEquals(metrics.SD_COVERAGE, 57.364434, .02);
             Assert.assertEquals(metrics.MEDIAN_COVERAGE, 0.0);
+            Assert.assertEquals(metrics.PCT_EXC_ADAPTER, 0.0);
             Assert.assertEquals(metrics.PCT_EXC_MAPQ, 0.0);
             Assert.assertEquals(metrics.PCT_EXC_UNPAIRED, 0.0);
             Assert.assertEquals(metrics.PCT_EXC_CAPPED, 0.519542, .001);
@@ -179,13 +176,14 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         }
 
         //Write SAM file
-        final SAMFileWriter writer = new SAMFileWriterFactory()
-                .setCreateIndex(true).makeBAMWriter(header, false, tempSamFileUnsorted);
 
-        for (final SAMRecord record : setBuilder) {
-            writer.addAlignment(record);
+        try (SAMFileWriter writer = new SAMFileWriterFactory()
+                .setCreateIndex(true).makeBAMWriter(header, false, tempSamFileUnsorted)) {
+
+            for (final SAMRecord record : setBuilder) {
+                writer.addAlignment(record);
+            }
         }
-        writer.close();
 
         //sort the temp file
         final SortSam sorter = new SortSam();
@@ -204,11 +202,12 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
     @DataProvider(name = "wgsAlgorithm")
     public Object[][] wgsAlgorithm() {
-        return new Object[][] {
+        return new Object[][]{
                 {"false"},
                 {"true"},
         };
     }
+
     @Test(dataProvider = "wgsAlgorithm")
     public void testLargeIntervals(final String useFastAlgorithm) throws IOException {
         final File input = new File(TEST_DIR, "forMetrics.sam");
@@ -217,8 +216,8 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         final File ref = new File(TEST_DIR, "merger.fasta");
         final File intervals = new File(TEST_DIR, "largeIntervals.interval_list");
         final int sampleSize = 1000;
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + ref.getAbsolutePath(),
                 "INTERVALS=" + intervals.getAbsolutePath(),
@@ -227,11 +226,13 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final MetricsFile<CollectWgsMetrics.WgsMetrics, Comparable<?>> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
-
-        for (final CollectWgsMetrics.WgsMetrics metrics : output.getMetrics()) {
+        final MetricsFile<WgsMetrics, Comparable<?>> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        for (final WgsMetrics metrics : output.getMetrics()) {
             Assert.assertEquals(metrics.GENOME_TERRITORY, 404);
+            Assert.assertEquals(metrics.PCT_EXC_ADAPTER, 0D);
             Assert.assertEquals(metrics.PCT_EXC_MAPQ, 0.271403);
             Assert.assertEquals(metrics.PCT_EXC_DUPE, 0.182149);
             Assert.assertEquals(metrics.PCT_EXC_UNPAIRED, 0.091075);
@@ -255,7 +256,7 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         expectedSingletonCoverage += 2 * 5; // 5 bases for each mate are good (see AAA!!!AA!! below).
         setBuilder.addPair("poorQualityReads", 1, 2, 20, false, false, "10M", "10M", true, false, -1);
 
-        for(int i = 1; i < 5; i++) {
+        for (int i = 1; i < 5; i++) {
             setBuilder.addPair("deepStack-" + i, 2, 2, 20, false, false, "10M", "10M", true, false, 30);
         }
 
@@ -267,20 +268,19 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         setBuilder.getSamReader();
 
         // Write SAM file
-        final SAMFileWriter writer = new SAMFileWriterFactory()
-                .setCreateIndex(true).makeBAMWriter(setBuilder.getHeader(), false, tempSamFile);
-
-        for (final SAMRecord record : setBuilder) {
-            writer.addAlignment(record);
+        try (SAMFileWriter writer = new SAMFileWriterFactory()
+                .setCreateIndex(true).makeBAMWriter(setBuilder.getHeader(), false, tempSamFile)) {
+            for (final SAMRecord record : setBuilder) {
+                writer.addAlignment(record);
+            }
         }
-        writer.close();
 
         // create output files for tests
         final File outfile = File.createTempFile("testWgsMetrics", ".txt");
         outfile.deleteOnExit();
 
-        final String[] args = new String[] {
-                "INPUT="  + tempSamFile.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + tempSamFile.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
                 "INCLUDE_BQ_HISTOGRAM=true",
@@ -289,21 +289,23 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final MetricsFile<CollectWgsMetrics.WgsMetrics, Integer> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
-        final CollectWgsMetrics.WgsMetrics metrics = output.getMetrics().get(0);
+        final MetricsFile<WgsMetrics, Integer> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        final WgsMetrics metrics = output.getMetrics().get(0);
 
         final Histogram<Integer> highQualityDepthHistogram = output.getAllHistograms().get(0);
         final Histogram<Integer> baseQHistogram = output.getAllHistograms().get(1);
 
         Assert.assertEquals((long) highQualityDepthHistogram.getSumOfValues(), metrics.GENOME_TERRITORY);
         Assert.assertEquals((long) highQualityDepthHistogram.get(1).getValue(), expectedSingletonCoverage);
-        Assert.assertEquals((long) highQualityDepthHistogram.get(3).getValue(), 2*10);
+        Assert.assertEquals((long) highQualityDepthHistogram.get(3).getValue(), 2 * 10);
     }
 
     @Test(dataProvider = "wgsAlgorithm")
     public void testPoorQualityBases(final String useFastAlgorithm) throws IOException {
-        final File reference = new File("testdata/picard/quality/chrM.reference.fasta");
+        final File reference = CHR_M_REFERENCE;
         final File testSamFile = VcfTestUtils.createTemporaryIndexedFile("CollectWgsMetrics", ".bam");
 
         /**
@@ -323,27 +325,25 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
         final SAMRecordSetBuilder setBuilder = CollectWgsMetricsTestUtils.createTestSAMBuilder(reference, READ_GROUP_ID, SAMPLE, PLATFORM, LIBRARY);
         setBuilder.setReadLength(10);
-        for (int i = 0; i < 3; i++){
+        for (int i = 0; i < 3; i++) {
             setBuilder.addPair("GreatBQRead:" + i, 0, 1, 30, false, false, "10M", "10M", false, true, 40);
         }
 
-        for (int i = 0; i < 3; i++){
+        for (int i = 0; i < 3; i++) {
             setBuilder.addPair("PoorBQRead:" + i, 0, 1, 30, false, false, "10M", "10M", false, true, 10);
         }
 
-        final SAMFileWriter writer = new SAMFileWriterFactory().setCreateIndex(true).makeBAMWriter(setBuilder.getHeader(), false, testSamFile);
-
-        for (final SAMRecord record : setBuilder) {
-            writer.addAlignment(record);
+        try (SAMFileWriter writer = new SAMFileWriterFactory().setCreateIndex(true).makeBAMWriter(setBuilder.getHeader(), false, testSamFile)) {
+            for (final SAMRecord record : setBuilder) {
+                writer.addAlignment(record);
+            }
         }
 
-        writer.close();
-
         final File outfile = File.createTempFile("testExcludedBases-metrics", ".txt");
         outfile.deleteOnExit();
 
-        final String[] args = new String[] {
-                "INPUT="  + testSamFile.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + testSamFile.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
                 "INCLUDE_BQ_HISTOGRAM=true",
@@ -353,10 +353,11 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final MetricsFile<CollectWgsMetrics.WgsMetrics, Integer> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
-
-        final CollectWgsMetrics.WgsMetrics metrics = output.getMetrics().get(0);
+        final MetricsFile<WgsMetrics, Integer> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        final WgsMetrics metrics = output.getMetrics().get(0);
 
         Assert.assertEquals(metrics.PCT_EXC_BASEQ, 0.5);
         Assert.assertEquals(metrics.PCT_EXC_CAPPED, 0.0);
@@ -364,28 +365,25 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
     @Test(dataProvider = "wgsAlgorithm")
     public void testGiantDeletion(final String useFastAlgorithm) throws IOException {
-        final File reference = new File("testdata/picard/quality/chrM.reference.fasta");
+        final File reference = CHR_M_REFERENCE;
         final File testSamFile = VcfTestUtils.createTemporaryIndexedFile("CollectWgsMetrics", ".bam");
 
         final SAMRecordSetBuilder setBuilder = CollectWgsMetricsTestUtils.createTestSAMBuilder(reference, READ_GROUP_ID, SAMPLE, PLATFORM, LIBRARY);
         setBuilder.setReadLength(10);
-        for (int i = 0; i < 3; i++){
+        for (int i = 0; i < 3; i++) {
             setBuilder.addPair("Read:" + i, 0, 1, 30, false, false, "5M10000D5M", "1000D10M", false, true, 40);
         }
 
-        final SAMFileWriter writer = new SAMFileWriterFactory().setCreateIndex(true).makeBAMWriter(setBuilder.getHeader(), false, testSamFile);
-
-        for (final SAMRecord record : setBuilder) {
-            writer.addAlignment(record);
+        try (SAMFileWriter writer = new SAMFileWriterFactory().setCreateIndex(true).makeBAMWriter(setBuilder.getHeader(), false, testSamFile)) {
+            for (final SAMRecord record : setBuilder) {
+                writer.addAlignment(record);
+            }
         }
-
-        writer.close();
-
         final File outfile = File.createTempFile("testGiantDeletion", ".txt");
         outfile.deleteOnExit();
 
-        final String[] args = new String[] {
-                "INPUT="  + testSamFile.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + testSamFile.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
                 "INCLUDE_BQ_HISTOGRAM=true",
@@ -395,12 +393,43 @@ public class CollectWgsMetricsTest extends CommandLineProgramTest {
 
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
-        final MetricsFile<CollectWgsMetrics.WgsMetrics, Integer> output = new MetricsFile<>();
-        output.read(new FileReader(outfile));
-
-        final CollectWgsMetrics.WgsMetrics metrics = output.getMetrics().get(0);
+        final MetricsFile<WgsMetrics, Integer> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        final WgsMetrics metrics = output.getMetrics().get(0);
 
         Assert.assertEquals(metrics.PCT_EXC_BASEQ, 0.0);
         Assert.assertEquals(metrics.PCT_EXC_CAPPED, 0.0);
     }
+
+    @Test(dataProvider = "wgsAlgorithm")
+    public void testAdapterReads(final String useFastAlgorithm) throws IOException {
+        final File metricsTestDir = new File(TEST_DIR.getParentFile(), "metrics");
+        final File input = new File(metricsTestDir, "AlignedAdapterReads.sam");
+        final File outfile = File.createTempFile("test", ".wgs_metrics");
+        outfile.deleteOnExit();
+        final String[] args = {
+                "INPUT=" + input.getAbsolutePath(),
+                "OUTPUT=" + outfile.getAbsolutePath(),
+                "SAMPLE_SIZE=" + 10,
+                "REFERENCE_SEQUENCE=" + CHR_M_REFERENCE.getAbsolutePath(),
+                "USE_FAST_ALGORITHM=" + useFastAlgorithm
+        };
+        Assert.assertEquals(runPicardCommandLine(args), 0);
+
+        final MetricsFile<WgsMetrics, Comparable<?>> output = new MetricsFile<>();
+
+        try (FileReader reader = new FileReader(outfile)) {
+            output.read(reader);
+        }
+        for (final WgsMetrics metrics : output.getMetrics()) {
+            Assert.assertEquals(metrics.GENOME_TERRITORY, 16571);
+            Assert.assertEquals(metrics.PCT_EXC_TOTAL, 1D);
+            TestNGUtil.compareDoubleWithAccuracy(metrics.PCT_EXC_ADAPTER, 102 / (102 + 82D), 0.00001);
+            TestNGUtil.compareDoubleWithAccuracy(metrics.PCT_EXC_MAPQ, 82 / (102 + 82D), 0.00001);
+            Assert.assertEquals(metrics.PCT_EXC_DUPE, 0.0);
+            Assert.assertEquals(metrics.PCT_EXC_UNPAIRED, 0.0);
+        }
+    }
 }
